@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA */
 
 /*
   Handling of uchar arrays as large bitmaps.
@@ -38,17 +38,6 @@
 #include <m_string.h>
 #include <my_bit.h>
 
-
-/* Create a mask of the significant bits for the last byte (1,3,7,..255) */
-
-static inline uchar last_byte_mask(uint bits)
-{
-  /* Get the number of used bits-1 (0..7) in the last byte */
-  unsigned int const used= (bits - 1U) & 7U;
-  /* Return bitmask for the significant bits */
-  return ((2U << used) - 1);
-}
-
 /*
   Create a mask with the upper 'unused' bits set and the lower 'used'
   bits clear. The bits within each byte is stored in big-endian order.
@@ -166,11 +155,15 @@ static inline uint get_first_set(my_bitmap_map value, uint word_pos)
   return MY_BIT_NONE;                           /* Impossible */
 }
 
+/*
+  Initialize a bitmap object. All bits will be set to zero
+*/
 
 my_bool my_bitmap_init(MY_BITMAP *map, my_bitmap_map *buf, uint n_bits,
                        my_bool thread_safe)
 {
   DBUG_ENTER("my_bitmap_init");
+  map->mutex= 0;
   if (!buf)
   {
     uint size_in_bytes= bitmap_buffer_size(n_bits);
@@ -180,7 +173,6 @@ my_bool my_bitmap_init(MY_BITMAP *map, my_bitmap_map *buf, uint n_bits,
       size_in_bytes= ALIGN_SIZE(size_in_bytes);
       extra= sizeof(mysql_mutex_t);
     }
-    map->mutex= 0;
     if (!(buf= (my_bitmap_map*) my_malloc(size_in_bytes+extra, MYF(MY_WME))))
       DBUG_RETURN(1);
     if (thread_safe)
