@@ -1,5 +1,5 @@
 /* Copyright (c) 2003, 2016, Oracle and/or its affiliates.
-   Copyright (c) 2011, 2020, MariaDB
+   Copyright (c) 2011, 2016, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -136,7 +136,7 @@ String *Item_func_geometry_from_json::val_str(String *str)
     {
       String *sv= args[1]->val_str(&tmp_js);
       my_error(ER_WRONG_VALUE_FOR_TYPE, MYF(0),
-               "option", sv->c_ptr_safe(), "ST_GeomFromGeoJSON");
+               "option", sv->c_ptr_safe(), "ST_GeometryFromJSON");
       null_value= 1;
       return 0;
     }
@@ -173,7 +173,7 @@ String *Item_func_geometry_from_json::val_str(String *str)
       code= ER_GEOJSON_NOT_CLOSED;
       break;
     case Geometry::GEOJ_DIMENSION_NOT_SUPPORTED:
-      my_error(ER_GIS_INVALID_DATA, MYF(0), "ST_GeomFromGeoJSON");
+      my_error(ER_GIS_INVALID_DATA, MYF(0), "ST_GeometryFromJSON");
       break;
     default:
       report_json_error_ex(js, &je, func_name(), 0, Sql_condition::WARN_LEVEL_WARN);
@@ -447,18 +447,16 @@ String *Item_func_boundary::val_str(String *str_value)
   DBUG_ASSERT(fixed == 1);
   String arg_val;
   String *swkb= args[0]->val_str(&arg_val);
-
-  if ((null_value= args[0]->null_value))
-    DBUG_RETURN(0);
-
   Geometry_buffer buffer;
+  Geometry *g;
   uint32 srid= 0;
   Transporter trn(&res_receiver);
-
-  Geometry *g= Geometry::construct(&buffer, swkb->ptr(), swkb->length());
-  if (!g)
+  
+  if ((null_value=
+       args[0]->null_value ||
+       !(g= Geometry::construct(&buffer, swkb->ptr(), swkb->length()))))
     DBUG_RETURN(0);
-
+  
   if (g->store_shapes(&trn))
     goto mem_error;
 
@@ -918,7 +916,7 @@ String *Item_func_point::val_str(String *str)
 
   if ((null_value= (args[0]->null_value ||
                     args[1]->null_value ||
-                    str->realloc(4/*SRID*/ + 1 + 4 + SIZEOF_STORED_DOUBLE * 2))))
+                    str->alloc(4/*SRID*/ + 1 + 4 + SIZEOF_STORED_DOUBLE * 2))))
     return 0;
 
   str->set_charset(&my_charset_bin);
@@ -2382,15 +2380,12 @@ double Item_func_distance::val_real()
   MBR mbr1, mbr2;
   const char *c_end;
 
-  if (args[0]->null_value || args[1]->null_value)
-    goto mem_error;
-  g1= Geometry::construct(&buffer1, res1->ptr(), res1->length());
-  if (!g1)
-    goto mem_error;
-  g2= Geometry::construct(&buffer2, res2->ptr(), res2->length());
-  if (!g2)
-    goto mem_error;
-  if (g1->get_mbr(&mbr1, &c_end) || g2->get_mbr(&mbr2, &c_end))
+
+  if ((null_value= (args[0]->null_value || args[1]->null_value ||
+          !(g1= Geometry::construct(&buffer1, res1->ptr(), res1->length())) ||
+          !(g2= Geometry::construct(&buffer2, res2->ptr(), res2->length())) ||
+          g1->get_mbr(&mbr1, &c_end) ||
+          g2->get_mbr(&mbr2, &c_end))))
     goto mem_error;
 
   mbr1.add_mbr(&mbr2);
@@ -2528,158 +2523,18 @@ mem_error:
 }
 
 
-double Item_func_sphere_distance::val_real()
-{
-  /* To test null_value of item, first get well-known bytes as a backups */
-  String bak1, bak2;
-  String *arg1= args[0]->val_str(&bak1);
-  String *arg2= args[1]->val_str(&bak2);
-  double distance= 0.0;
-  double sphere_radius= 6370986.0; // Default radius equals Earth radius
-  
-  null_value= (args[0]->null_value || args[1]->null_value);
-  if (null_value)
-  {
-    return 0;
-  }
-
-  if (arg_count == 3)
-  {
-    sphere_radius= args[2]->val_real();
-    // Radius cannot be Null
-    if (args[2]->null_value)
-    {
-      null_value= true;
-      return 0;
-    }
-    if (sphere_radius <= 0)
-    {
-      my_error(ER_INTERNAL_ERROR, MYF(0), "Radius must be greater than zero.");
-      return 1;
-    }
-  }
-  Geometry_buffer buffer1, buffer2;
-  Geometry *g1, *g2;
-  if (!(g1= Geometry::construct(&buffer1, arg1->ptr(), arg1->length())) ||
-      !(g2= Geometry::construct(&buffer2, arg2->ptr(), arg2->length())))
-  {
-    my_error(ER_GIS_INVALID_DATA, MYF(0), "ST_Distance_Sphere");
-    goto handle_errors;
-  }
-// Method allowed for points and multipoints
-  if (!(g1->get_class_info()->m_type_id == Geometry::wkb_point ||
-        g1->get_class_info()->m_type_id == Geometry::wkb_multipoint) ||
-      !(g2->get_class_info()->m_type_id == Geometry::wkb_point ||
-        g2->get_class_info()->m_type_id == Geometry::wkb_multipoint))
-  {
-    // Generate error message in case different geometry is used? 
-    my_error(ER_INTERNAL_ERROR, MYF(0), func_name());
-    return 0;
-  }
-  distance= spherical_distance_points(g1, g2, sphere_radius);
-  if (distance < 0)
-  {
-    my_error(ER_INTERNAL_ERROR, MYF(0), "Returned distance cannot be negative.");
-    return 1;
-  }
-  return distance;
-
-  handle_errors:
-    return 0;
-}
-
-
-double Item_func_sphere_distance::spherical_distance_points(Geometry *g1,
-                                                            Geometry *g2,
-                                                            const double r)
-{
-  double res= 0.0;
-   // Length for the single point (25 Bytes)
-  uint32 len= SRID_SIZE + POINT_DATA_SIZE + WKB_HEADER_SIZE;
-  int err_hv= 0, err_sph= 0;
-
-  switch (g2->get_class_info()->m_type_id)
-  {
-    case Geometry::wkb_point:
-    {
-      Gis_point *g2p= static_cast<Gis_point *>(g2);
-      // Optimization for point-point case
-      if (g1->get_class_info()->m_type_id == Geometry::wkb_point)
-      {
-        res= g2p->calculate_haversine(g1, r, &err_hv);
-      }
-      else
-      {
-        // Optimization for single point in Multipoint
-        if (g1->get_data_size() == len)
-        {
-          res= g2p->calculate_haversine(g1, r, &err_hv);
-        }
-        else
-        {
-          // There are multipoints in g1
-          // g1 is MultiPoint and calculate MP.sphericaldistance from g2 Point
-          if (g1->get_data_size() != GET_SIZE_ERROR)
-            err_sph= g2p->spherical_distance_multipoints(g1, r, &res, &err_hv);
-        }
-      }
-      break;
-    }
-
-    case Geometry::wkb_multipoint:
-      // Optimization for point-point case
-      if (g1->get_class_info()->m_type_id == Geometry::wkb_point)
-      {
-        Gis_point *g1p= static_cast<Gis_point *>(g1);
-         // Optimization for single point in Multipoint g2
-        if (g2->get_data_size() == len)
-        {
-          res= g1p->calculate_haversine(g2, r, &err_hv);
-        }
-        else
-        {
-          if (g2->get_data_size() != GET_SIZE_ERROR)
-          // g1 is a point (casted to multi_point) and g2 multipoint
-            err_sph= g1p->spherical_distance_multipoints(g2, r, &res, &err_hv);
-        }
-      }
-      else
-      {
-        Gis_multi_point *g1mp= static_cast<Gis_multi_point *>(g1);
-        // Multipoints in g1 and g2 - no optimization
-        err_sph= g1mp->spherical_distance_multipoints(g2, r, &res, &err_hv);
-      }
-      break;
-
-    default:
-      DBUG_ASSERT(0);
-      break;
-  }
-
-  if (err_hv == 1)
-    my_error(ER_STD_OUT_OF_RANGE_ERROR, MYF(0),
-             "Longitude should be [-180,180]", "ST_Distance_Sphere");
-  else if(err_hv < 0)
-    my_error(ER_STD_OUT_OF_RANGE_ERROR, MYF(0),
-             "Latitude should be [-90,90]", "ST_Distance_Sphere");
-  else if (err_sph || err_hv == 2)
-    my_error(ER_CANT_CREATE_GEOMETRY_OBJECT, MYF(0));
-  return res;
-}
-
-
 String *Item_func_pointonsurface::val_str(String *str)
 {
   Gcalc_operation_transporter trn(&func, &collector);
 
-  DBUG_ENTER("Item_func_pointonsurface::val_str");
+  DBUG_ENTER("Item_func_pointonsurface::val_real");
   DBUG_ASSERT(fixed == 1);
   String *res= args[0]->val_str(&tmp_value);
   Geometry_buffer buffer;
   Geometry *g;
   MBR mbr;
   const char *c_end;
-  double UNINIT_VAR(px), UNINIT_VAR(py), x0, UNINIT_VAR(y0);
+  double UNINIT_VAR(px), UNINIT_VAR(py), x0, y0;
   String *result= 0;
   const Gcalc_scan_iterator::point *pprev= NULL;
   uint32 srid;
