@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301 USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 /*
   mysql_upgrade_service upgrades mysql service on Windows.
@@ -134,7 +134,7 @@ static void die(const char *fmt, ...)
   }
 
   /*
-    Stop service that we started, if it was not initally running at
+    Stop service that we started, if it was not initially running at
     program start.
   */
   if (initial_service_state != UINT_MAX && initial_service_state != SERVICE_RUNNING)
@@ -496,7 +496,7 @@ int main(int argc, char **argv)
     old_mysqld_exe_exists?",this can take some time":"(skipped)");
 
   char socket_param[FN_REFLEN];
-  sprintf_s(socket_param, "--socket=mysql_upgrade_service_%d",
+  sprintf_s(socket_param, "--socket=mysql_upgrade_service_%u",
     GetCurrentProcessId());
 
   DWORD start_duration_ms = 0;
@@ -513,7 +513,7 @@ int main(int argc, char **argv)
       die("Cannot start mysqld.exe process, last error =%u", GetLastError());
     }
     char pipe_name[64];
-    snprintf(pipe_name, sizeof(pipe_name), "\\\\.\\pipe\\mysql_upgrade_service_%u",
+    snprintf(pipe_name, sizeof(pipe_name), "\\\\.\\pipe\\mysql_upgrade_service_%lu",
       GetCurrentProcessId());
     for (;;)
     {
