@@ -1,5 +1,5 @@
 /* Copyright (c) 2000, 2017, Oracle and/or its affiliates.
-   Copyright (c) 2010, 2020, MariaDB Corporation.
+   Copyright (c) 2010, 2017, Corporation
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,7 +30,6 @@
 #include "filesort.h"                           // filesort_free_buffers
 #include "sql_view.h"
 #include "sql_cte.h"
-#include "item_windowfunc.h"
 
 bool mysql_union(THD *thd, LEX *lex, select_result *result,
                  SELECT_LEX_UNIT *unit, ulong setup_tables_done_option)
@@ -69,7 +68,7 @@ void select_unit::change_select()
   curr_sel= current_select_number;
   /* New SELECT processing starts */
   DBUG_ASSERT(table->file->inited == 0);
-  step= thd->lex->current_select->linkage;
+  step= thd->lex->current_select->get_linkage();
   switch (step)
   {
   case INTERSECT_TYPE:
@@ -249,7 +248,7 @@ bool select_unit::send_eof()
 {
   if (step != INTERSECT_TYPE ||
       (thd->lex->current_select->next_select() &&
-       thd->lex->current_select->next_select()->linkage == INTERSECT_TYPE))
+       thd->lex->current_select->next_select()->get_linkage() == INTERSECT_TYPE))
   {
     /*
       it is not INTESECT or next SELECT in the sequence is INTERSECT so no
@@ -297,27 +296,7 @@ bool select_unit::send_eof()
 
 int select_union_recursive::send_data(List<Item> &values)
 {
-
-  int rc;
-  bool save_abort_on_warning= thd->abort_on_warning;
-  enum_check_fields save_count_cuted_fields= thd->count_cuted_fields;
-  long save_counter;
-
-  /*
-    For recursive CTE's give warnings for wrong field info
-    However, we don't do that for CREATE TABLE ... SELECT or INSERT ... SELECT
-    as the upper level code for these handles setting of abort_on_warning
-    depending on if 'IGNORE' is used.
-  */
-  if (thd->lex->sql_command != SQLCOM_CREATE_TABLE &&
-      thd->lex->sql_command != SQLCOM_INSERT_SELECT)
-    thd->abort_on_warning= thd->is_strict_mode();
-  thd->count_cuted_fields= CHECK_FIELD_WARN;
-  save_counter= thd->get_stmt_da()->set_current_row_for_warning(++row_counter);
-  rc= select_unit::send_data(values);
-  thd->get_stmt_da()->set_current_row_for_warning(save_counter);
-  thd->count_cuted_fields= save_count_cuted_fields;
-  thd->abort_on_warning= save_abort_on_warning;
+  int rc= select_unit::send_data(values);
 
   if (rc == 0 &&
       write_err != HA_ERR_FOUND_DUPP_KEY &&
@@ -426,10 +405,7 @@ select_union_recursive::create_result_table(THD *thd_arg,
                                        hidden))
     return true;
   
-  incr_table_param.init();
-  incr_table_param.field_count= column_types->elements;
-  incr_table_param.bit_fields_as_long= bit_fields_as_long;
-  if (! (incr_table= create_tmp_table(thd_arg, &incr_table_param, *column_types,
+  if (! (incr_table= create_tmp_table(thd_arg, &tmp_table_param, *column_types,
                                       (ORDER*) 0, false, 1,
                                       options, HA_POS_ERROR, &empty_clex_str,
                                       true, keep_row_order)))
@@ -439,6 +415,20 @@ select_union_recursive::create_result_table(THD *thd_arg,
   for (uint i=0; i < table->s->fields; i++)
     incr_table->field[i]->flags &= ~(PART_KEY_FLAG | PART_INDIRECT_KEY_FLAG);
 
+  TABLE *rec_table= 0;
+  if (! (rec_table= create_tmp_table(thd_arg, &tmp_table_param, *column_types,
+                                     (ORDER*) 0, false, 1,
+                                     options, HA_POS_ERROR, alias,
+                                     true, keep_row_order)))
+    return true;
+
+  rec_table->keys_in_use_for_query.clear_all();
+  for (uint i=0; i < table->s->fields; i++)
+    rec_table->field[i]->flags &= ~(PART_KEY_FLAG | PART_INDIRECT_KEY_FLAG);
+
+  if (rec_tables.push_back(rec_table))
+    return true;
+
   return false;
 }
 
@@ -476,27 +466,24 @@ void select_union_recursive::cleanup()
     free_tmp_table(thd, incr_table);
   }
 
-  List_iterator<TABLE_LIST> it(rec_table_refs);
-  TABLE_LIST *tbl;
-  while ((tbl= it++))
+  List_iterator<TABLE> it(rec_tables);
+  TABLE *tab;
+  while ((tab= it++))
   {
-    TABLE *tab= tbl->table;
     if (tab->is_created())
     {
       tab->file->extra(HA_EXTRA_RESET_STATE);
       tab->file->ha_delete_all_rows();
     }
-    /*
+    /* 
       The table will be closed later in close_thread_tables(),
       because it might be used in the statements like
       ANALYZE WITH r AS (...) SELECT * from r
-      where r is defined through recursion.
+      where r is defined through recursion. 
     */
     tab->next= thd->rec_tables;
     thd->rec_tables= tab;
-    tbl->derived_result= 0;
   }
-  row_counter= 0;
 }
 
 
@@ -633,6 +620,15 @@ st_select_lex_unit::init_prepare_fake_select_lex(THD *thd_arg,
          order= order->next)
       order->item= &order->item_ptr;
   }
+  for (ORDER *order= global_parameters()->order_list.first;
+       order;
+       order=order->next)
+  {
+    (*order->item)->walk(&Item::change_context_processor, 0,
+                         &fake_select_lex->context);
+    (*order->item)->walk(&Item::set_fake_select_as_master_processor, 0,
+                         fake_select_lex);
+  }
 }
 
 
@@ -757,11 +753,11 @@ bool st_select_lex_unit::join_union_type_attributes(THD *thd_arg,
         been fixed yet. An Item_type_holder must be created based on a fixed
         Item, so use the inner Item instead.
       */
-      DBUG_ASSERT(item_tmp->fixed ||
+      DBUG_ASSERT(item_tmp->is_fixed() ||
                   (item_tmp->type() == Item::REF_ITEM &&
                    ((Item_ref *)(item_tmp))->ref_type() ==
                    Item_ref::OUTER_REF));
-      if (!item_tmp->fixed)
+      if (!item_tmp->is_fixed())
         item_tmp= item_tmp->real_item();
       holders[holder_pos].add_argument(item_tmp);
     }
@@ -835,8 +831,8 @@ bool st_select_lex_unit::prepare(TABLE_LIST *derived_arg,
   bool is_union_select;
   bool have_except= FALSE, have_intersect= FALSE;
   bool instantiate_tmp_table= false;
-  bool single_tvc= !first_sl->next_select() && first_sl->tvc &&
-                   !fake_select_lex;
+  bool single_tvc= !first_sl->next_select() && first_sl->tvc;
+  bool single_tvc_wo_order= single_tvc && !first_sl->order_list.elements;
   DBUG_ENTER("st_select_lex_unit::prepare");
   DBUG_ASSERT(thd == current_thd);
 
@@ -910,25 +906,6 @@ bool st_select_lex_unit::prepare(TABLE_LIST *derived_arg,
   found_rows_for_union= first_sl->options & OPTION_FOUND_ROWS;
   is_union_select= is_unit_op() || fake_select_lex || single_tvc;
 
-  /*
-    If we are reading UNION output and the UNION is in the
-    IN/ANY/ALL/EXISTS subquery, then ORDER BY is redundant and hence should
-    be removed.
-    Example:
-     select ... col IN (select col2 FROM t1 union select col3 from t2 ORDER BY 1)
-
-    (as for ORDER BY ... LIMIT, it currently not supported inside
-     IN/ALL/ANY subqueries)
-    (For non-UNION this removal of ORDER BY clause is done in
-     check_and_do_in_subquery_rewrites())
-  */
-  if (item && is_unit_op() &&
-      (item->is_in_predicate() || item->is_exists_predicate()))
-  {
-    global_parameters()->order_list.first= NULL;
-    global_parameters()->order_list.elements= 0;
-  }
-
   for (SELECT_LEX *s= first_sl; s; s= s->next_select())
   {
     switch (s->linkage)
@@ -943,13 +920,13 @@ bool st_select_lex_unit::prepare(TABLE_LIST *derived_arg,
       break;
     }
   }
-
   /* Global option */
 
   if (is_union_select || is_recursive)
   {
-    if ((is_unit_op() && !union_needs_tmp_table() &&
-        !have_except && !have_intersect) || single_tvc)
+    if ((single_tvc_wo_order && !fake_select_lex) ||
+        (is_unit_op() && !union_needs_tmp_table() &&
+	 !have_except && !have_intersect && !single_tvc))
     {
       SELECT_LEX *last= first_select();
       while (last->next_select())
@@ -964,7 +941,7 @@ bool st_select_lex_unit::prepare(TABLE_LIST *derived_arg,
     else
     {
       if (!is_recursive)
-	union_result= new (thd->mem_root) select_unit(thd);
+       union_result= new (thd->mem_root) select_unit(thd);
       else
       {
         with_element->rec_result=
@@ -1018,7 +995,47 @@ bool st_select_lex_unit::prepare(TABLE_LIST *derived_arg,
     types= first_sl->item_list;
     goto cont;
   }
- 
+
+  if (sl->tvc && sl->order_list.elements &&
+      !sl->tvc->to_be_wrapped_as_with_tail())
+  {
+    SELECT_LEX_UNIT *unit= sl->master_unit();
+    if (thd->lex->context_analysis_only & CONTEXT_ANALYSIS_ONLY_VIEW)
+    {
+      unit->fake_select_lex= 0;
+      unit->saved_fake_select_lex= 0;
+    }
+    else
+    {
+      if (!unit->first_select()->next_select())
+      {
+        if (!unit->fake_select_lex)
+	{
+          Query_arena *arena, backup_arena;
+          arena= thd->activate_stmt_arena_if_needed(&backup_arena);
+          bool rc= unit->add_fake_select_lex(thd);
+          if (arena)
+            thd->restore_active_arena(arena, &backup_arena);
+          if (rc)
+            goto err;
+        }
+        SELECT_LEX *fake= unit->fake_select_lex;
+        fake->order_list= sl->order_list;
+        fake->explicit_limit= sl->explicit_limit;
+        fake->select_limit= sl->select_limit;
+        fake->offset_limit= sl->offset_limit;
+        sl->order_list.empty();
+        sl->explicit_limit= 0;
+        sl->select_limit= 0;
+        sl->offset_limit= 0;
+        if (describe)
+          fake->options|= SELECT_DESCRIBE;
+      }
+      else if (!sl->explicit_limit)
+        sl->order_list.empty();
+    }
+  }
+
   for (;sl; sl= sl->next_select(), union_part_count++)
   {
     if (sl->tvc)
@@ -1040,7 +1057,7 @@ bool st_select_lex_unit::prepare(TABLE_LIST *derived_arg,
 	  goto err;
       }
       else if (sl->tvc->prepare(thd, sl, tmp_result, this))
-	goto err;
+	  goto err;
     }
     else if (prepare_join(thd, sl, tmp_result, additional_options,
                           is_union_select))
@@ -1101,33 +1118,9 @@ bool st_select_lex_unit::prepare(TABLE_LIST *derived_arg,
           goto err;
         if (!derived_arg->table)
         {
-          bool res= false;
-
-          if ((!derived_arg->is_with_table_recursive_reference() ||
-               !derived_arg->derived_result) &&
-              !(derived_arg->derived_result=
-                new (thd->mem_root) select_unit(thd)))
-            goto err; // out of memory
-          thd->create_tmp_table_for_derived= TRUE;
-
-          res= derived_arg->derived_result->create_result_table(thd,
-                                                            &types,
-                                                            FALSE,
-                                                            create_options,
-                                                            &derived_arg->alias,
-                                                            FALSE, FALSE,
-                                                            FALSE, 0);
-          thd->create_tmp_table_for_derived= FALSE;
-          if (res)
-            goto err;
-          derived_arg->derived_result->set_unit(this);
-          derived_arg->table= derived_arg->derived_result->table;
-          if (derived_arg->is_with_table_recursive_reference())
-          {
-            /* Here 'derived_arg' is the primary recursive table reference */
-            derived_arg->with->rec_result->
-              rec_table_refs.push_back(derived_arg);
-          }
+          derived_arg->table= with_element->rec_result->rec_tables.head();
+          if (derived_arg->derived_result)
+            derived_arg->derived_result->table= derived_arg->table;
         }
         with_element->mark_as_with_prepared_anchor();
         is_rec_result_table_created= true;
@@ -1166,8 +1159,7 @@ cont:
         Test if the aggregated data type is OK for a UNION element.
         E.g. in case of string data, DERIVATION_NONE is not allowed.
       */
-      if (type->type() == Item::TYPE_HOLDER && type->type_handler()->
-            union_element_finalize(static_cast<Item_type_holder*>(type)))
+      if (type->type_handler()->union_element_finalize(type))
         goto err;
     }
     
@@ -1510,7 +1502,7 @@ bool st_select_lex_unit::exec()
         union_result->change_select();
       if (fake_select_lex)
       {
-        if (sl != &thd->lex->select_lex)
+        if (sl != thd->lex->first_select_lex())
           fake_select_lex->uncacheable|= sl->uncacheable;
         else
           fake_select_lex->uncacheable= 0;
@@ -1771,11 +1763,11 @@ bool st_select_lex_unit::exec_recursive()
   TABLE *incr_table= with_element->rec_result->incr_table;
   st_select_lex *end= NULL;
   bool is_unrestricted= with_element->is_unrestricted();
-  List_iterator_fast<TABLE_LIST> li(with_element->rec_result->rec_table_refs);
+  List_iterator_fast<TABLE> li(with_element->rec_result->rec_tables);
   TMP_TABLE_PARAM *tmp_table_param= &with_element->rec_result->tmp_table_param;
   ha_rows examined_rows= 0;
   bool was_executed= executed;
-  TABLE_LIST *rec_tbl;
+  TABLE *rec_table;
 
   DBUG_ENTER("st_select_lex_unit::exec_recursive");
 
@@ -1853,9 +1845,8 @@ bool st_select_lex_unit::exec_recursive()
   else
     with_element->level++;
 
-  while ((rec_tbl= li++))
+  while ((rec_table= li++))
   {
-    TABLE *rec_table= rec_tbl->table;
     saved_error=
       incr_table->insert_all_rows_into_tmp_table(thd, rec_table,
                                                  tmp_table_param,
@@ -1889,8 +1880,13 @@ bool st_select_lex_unit::cleanup()
   {
     DBUG_RETURN(FALSE);
   }
-  if (with_element && with_element->is_recursive && union_result &&
-      with_element->rec_outer_references)
+  /*
+    When processing a PS/SP or an EXPLAIN command cleanup of a unit can
+    be performed immediately when the unit is reached in the cleanup
+    traversal initiated by the cleanup of the main unit.
+  */
+  if (!thd->stmt_arena->is_stmt_prepare() && !thd->lex->describe &&
+      with_element && with_element->is_recursive && union_result)
   {
     select_union_recursive *result= with_element->rec_result;
     if (++result->cleanup_count == with_element->rec_outer_references)
@@ -2059,29 +2055,6 @@ static void cleanup_order(ORDER *order)
 }
 
 
-static void cleanup_window_funcs(List<Item_window_func> &win_funcs)
-{
-  List_iterator_fast<Item_window_func> it(win_funcs);
-  Item_window_func *win_func;
-  while ((win_func= it++))
-  {
-    Window_spec *win_spec= win_func->window_spec;
-    if (!win_spec)
-      continue;
-    if (win_spec->save_partition_list)
-    {
-      win_spec->partition_list= win_spec->save_partition_list;
-      win_spec->save_partition_list= NULL;
-    }
-    if (win_spec->save_order_list)
-    {
-      win_spec->order_list= win_spec->save_order_list;
-      win_spec->save_order_list= NULL;
-    }
-  }
-}
-
-
 bool st_select_lex::cleanup()
 {
   bool error= FALSE;
@@ -2091,37 +2064,16 @@ bool st_select_lex::cleanup()
   cleanup_order(group_list.first);
   cleanup_ftfuncs(this);
 
-  cleanup_window_funcs(window_funcs);
-
   if (join)
   {
-    List_iterator<TABLE_LIST> ti(leaf_tables);
-    TABLE_LIST *tbl;
-    while ((tbl= ti++))
-    {
-      if (tbl->is_recursive_with_table() &&
-          !tbl->is_with_table_recursive_reference())
-      {
-        /*
-          If query is killed before open_and_process_table() for tbl
-          is called then 'with' is already set, but 'derived' is not.
-        */
-        st_select_lex_unit *unit= tbl->with->spec;
-        error|= (bool) error | (uint) unit->cleanup();
-      }
-    }
     DBUG_ASSERT((st_select_lex*)join->select_lex == this);
     error= join->destroy();
     delete join;
     join= 0;
   }
-  leaf_tables.empty();
   for (SELECT_LEX_UNIT *lex_unit= first_inner_unit(); lex_unit ;
        lex_unit= lex_unit->next_unit())
   {
-    if (lex_unit->with_element && lex_unit->with_element->is_recursive &&
-        lex_unit->with_element->rec_outer_references)
-      continue;
     error= (bool) ((uint) error | (uint) lex_unit->cleanup());
   }
   inner_refs_list.empty();
@@ -2141,12 +2093,8 @@ void st_select_lex::cleanup_all_joins(bool full)
     join->cleanup(full);
 
   for (unit= first_inner_unit(); unit; unit= unit->next_unit())
-  {
-    if (unit->with_element && unit->with_element->is_recursive)
-      continue;
     for (sl= unit->first_select(); sl; sl= sl->next_select())
       sl->cleanup_all_joins(full);
-  }
   DBUG_VOID_RETURN;
 }
 
