@@ -45,13 +45,13 @@ public:
   {
     m_thd->variables.option_bits&= ~OPTION_BEGIN;
     m_thd->server_status&= ~SERVER_STATUS_IN_TRANS;
-    m_thd->wsrep_cs().enter_toi(ws_meta);
+    m_thd->wsrep_cs().enter_toi_mode(ws_meta);
   }
   ~Wsrep_non_trans_mode()
   {
     m_thd->variables.option_bits= m_option_bits;
     m_thd->server_status= m_server_status;
-    m_thd->wsrep_cs().leave_toi();
+    m_thd->wsrep_cs().leave_toi_mode();
   }
 private:
   Wsrep_non_trans_mode(const Wsrep_non_trans_mode&);
@@ -205,6 +205,12 @@ const wsrep::transaction& Wsrep_high_priority_service::transaction() const
   DBUG_RETURN(m_thd->wsrep_trx());
 }
 
+int Wsrep_high_priority_service::next_fragment(const wsrep::ws_meta& ws_meta)
+{
+  DBUG_ENTER(" Wsrep_high_priority_service::next_fragment");
+  DBUG_RETURN(m_thd->wsrep_cs().next_fragment(ws_meta));
+}
+
 int Wsrep_high_priority_service::adopt_transaction(
   const wsrep::transaction& transaction)
 {
@@ -225,7 +231,8 @@ int Wsrep_high_priority_service::adopt_transaction(
 int Wsrep_high_priority_service::append_fragment_and_commit(
   const wsrep::ws_handle& ws_handle,
   const wsrep::ws_meta& ws_meta,
-  const wsrep::const_buffer& data)
+  const wsrep::const_buffer& data,
+  const wsrep::xid& xid WSREP_UNUSED)
 {
   DBUG_ENTER("Wsrep_high_priority_service::append_fragment_and_commit");
   int ret= start_transaction(ws_handle, ws_meta);
@@ -343,7 +350,8 @@ int Wsrep_high_priority_service::rollback(const wsrep::ws_handle& ws_handle,
 }
 
 int Wsrep_high_priority_service::apply_toi(const wsrep::ws_meta& ws_meta,
-                                           const wsrep::const_buffer& data)
+                                           const wsrep::const_buffer& data,
+                                           wsrep::mutable_buffer&)
 {
   DBUG_ENTER("Wsrep_high_priority_service::apply_toi");
   THD* thd= m_thd;
@@ -378,20 +386,13 @@ int Wsrep_high_priority_service::apply_toi(const wsrep::ws_meta& ws_meta,
 
 void Wsrep_high_priority_service::store_globals()
 {
-  DBUG_ENTER("Wsrep_high_priority_service::store_globals");
-  /* In addition to calling THD::store_globals(), call
-     wsrep::client_state::store_globals() to gain ownership of
-     the client state */
-  m_thd->store_globals();
-  m_thd->wsrep_cs().store_globals();
-  DBUG_VOID_RETURN;
+  wsrep_store_threadvars(m_thd);
+  m_thd->wsrep_cs().acquire_ownership();
 }
 
 void Wsrep_high_priority_service::reset_globals()
 {
-  DBUG_ENTER("Wsrep_high_priority_service::reset_globals");
-  m_thd->reset_globals();
-  DBUG_VOID_RETURN;
+  wsrep_reset_threadvars(m_thd);
 }
 
 void Wsrep_high_priority_service::switch_execution_context(wsrep::high_priority_service& orig_high_priority_service)
@@ -404,21 +405,34 @@ void Wsrep_high_priority_service::switch_execution_context(wsrep::high_priority_
 }
 
 int Wsrep_high_priority_service::log_dummy_write_set(const wsrep::ws_handle& ws_handle,
-                                                     const wsrep::ws_meta& ws_meta)
+                                                     const wsrep::ws_meta& ws_meta,
+                                                     wsrep::mutable_buffer& err)
 {
   DBUG_ENTER("Wsrep_high_priority_service::log_dummy_write_set");
   int ret= 0;
   DBUG_PRINT("info",
              ("Wsrep_high_priority_service::log_dummy_write_set: seqno=%lld",
               ws_meta.seqno().get()));
-  m_thd->wsrep_cs().start_transaction(ws_handle, ws_meta);
-  WSREP_DEBUG("Log dummy write set %lld", ws_meta.seqno().get());
-  if (!(opt_log_slave_updates && wsrep_gtid_mode && m_thd->variables.gtid_seq_no))
+  if (ws_meta.ordered())
   {
-    m_thd->wsrep_cs().before_rollback();
-    m_thd->wsrep_cs().after_rollback();
+    wsrep::client_state& cs(m_thd->wsrep_cs());
+    if (!cs.transaction().active())
+    {
+      cs.start_transaction(ws_handle, ws_meta);
+    }
+    adopt_apply_error(err);
+    WSREP_DEBUG("Log dummy write set %lld", ws_meta.seqno().get());
+    ret= cs.provider().commit_order_enter(ws_handle, ws_meta);
+    if (!(ret && opt_log_slave_updates && wsrep_gtid_mode &&
+          m_thd->variables.gtid_seq_no))
+    {
+      cs.before_rollback();
+      cs.after_rollback();
+    }
+    wsrep_set_SE_checkpoint(ws_meta.gtid());
+    ret= ret || cs.provider().commit_order_leave(ws_handle, ws_meta, err);
+    cs.after_applying();
   }
-  m_thd->wsrep_cs().after_applying();
   DBUG_RETURN(ret);
 }
 
@@ -439,11 +453,15 @@ Wsrep_applier_service::Wsrep_applier_service(THD* thd)
   thd->wsrep_cs().open(wsrep::client_id(thd->thread_id));
   thd->wsrep_cs().before_command();
   thd->wsrep_cs().debug_log_level(wsrep_debug);
-
+  if (!thd->slave_thread)
+    thd->system_thread_info.rpl_sql_info=
+      new rpl_sql_thread_info(thd->wsrep_rgi->rli->mi->rpl_filter);
 }
 
 Wsrep_applier_service::~Wsrep_applier_service()
 {
+  if (!m_thd->slave_thread)
+    delete m_thd->system_thread_info.rpl_sql_info;
   m_thd->wsrep_cs().after_command_before_result();
   m_thd->wsrep_cs().after_command_after_result();
   m_thd->wsrep_cs().close();
@@ -452,7 +470,8 @@ Wsrep_applier_service::~Wsrep_applier_service()
 }
 
 int Wsrep_applier_service::apply_write_set(const wsrep::ws_meta& ws_meta,
-                                           const wsrep::const_buffer& data)
+                                           const wsrep::const_buffer& data,
+                                           wsrep::mutable_buffer&)
 {
   DBUG_ENTER("Wsrep_applier_service::apply_write_set");
   THD* thd= m_thd;
@@ -495,6 +514,14 @@ int Wsrep_applier_service::apply_write_set(const wsrep::ws_meta& ws_meta,
   DBUG_RETURN(ret);
 }
 
+int Wsrep_applier_service::apply_nbo_begin(const wsrep::ws_meta& ws_meta,
+                                           const wsrep::const_buffer& data,
+                                           wsrep::mutable_buffer& err)
+{
+  DBUG_ENTER("Wsrep_applier_service::apply_nbo_begin");
+  DBUG_RETURN(0);
+}
+
 void Wsrep_applier_service::after_apply()
 {
   DBUG_ENTER("Wsrep_applier_service::after_apply");
@@ -558,11 +585,14 @@ Wsrep_replayer_service::Wsrep_replayer_service(THD* replayer_thd, THD* orig_thd)
   thd_proc_info(orig_thd, "wsrep replaying trx");
 
   /*
-    Swith execution context to replayer_thd and prepare it for
+    Switch execution context to replayer_thd and prepare it for
     replay execution.
   */
-  orig_thd->reset_globals();
-  replayer_thd->store_globals();
+  /* Copy thd vars from orig_thd before reset, otherwise reset
+     for orig thd clears thread local storage before copy. */
+  wsrep_assign_from_threadvars(replayer_thd);
+  wsrep_reset_threadvars(orig_thd);
+  wsrep_store_threadvars(replayer_thd);
   wsrep_open(replayer_thd);
   wsrep_before_command(replayer_thd);
   replayer_thd->wsrep_cs().clone_transaction_for_replay(orig_thd->wsrep_trx());
@@ -573,14 +603,12 @@ Wsrep_replayer_service::~Wsrep_replayer_service()
   THD* replayer_thd= m_thd;
   THD* orig_thd= m_orig_thd;
 
-  /* Store replay result/state to original thread wsrep client
-     state and switch execution context back to original. */
-  orig_thd->wsrep_cs().after_replay(replayer_thd->wsrep_trx());
+  /* Switch execution context back to original. */
   wsrep_after_apply(replayer_thd);
   wsrep_after_command_ignore_result(replayer_thd);
   wsrep_close(replayer_thd);
-  replayer_thd->reset_globals();
-  orig_thd->store_globals();
+  wsrep_reset_threadvars(replayer_thd);
+  wsrep_store_threadvars(orig_thd);
 
   DBUG_ASSERT(!orig_thd->get_stmt_da()->is_sent());
   DBUG_ASSERT(!orig_thd->get_stmt_da()->is_set());
@@ -588,7 +616,7 @@ Wsrep_replayer_service::~Wsrep_replayer_service()
   if (m_replay_status == wsrep::provider::success)
   {
     DBUG_ASSERT(replayer_thd->wsrep_cs().current_error() == wsrep::e_success);
-    orig_thd->killed= NOT_KILLED;
+    orig_thd->reset_kill_query();
     my_ok(orig_thd, m_da_shadow.affected_rows, m_da_shadow.last_insert_id);
   }
   else if (m_replay_status == wsrep::provider::error_certification_failed)
@@ -606,7 +634,8 @@ Wsrep_replayer_service::~Wsrep_replayer_service()
 }
 
 int Wsrep_replayer_service::apply_write_set(const wsrep::ws_meta& ws_meta,
-                                                 const wsrep::const_buffer& data)
+                                            const wsrep::const_buffer& data,
+                                            wsrep::mutable_buffer&)
 {
   DBUG_ENTER("Wsrep_replayer_service::apply_write_set");
   THD* thd= m_thd;
