@@ -73,7 +73,8 @@ PGLOBAL CntExit(PGLOBAL g)
 			g->Activityp = NULL;
 		}	// endif Activityp
 
-    g= PlugExit(g);
+    PlugExit(g);
+    g= NULL;
     } // endif g
 
   return g;
@@ -148,7 +149,7 @@ bool CntCheckDB(PGLOBAL g, PHC handler, const char *pathname)
   /*********************************************************************/
   /*  All is correct.                                                  */
   /*********************************************************************/
-  snprintf(g->Message, sizeof(g->Message), MSG(DATABASE_LOADED), "???");
+  sprintf(g->Message, MSG(DATABASE_LOADED), "???");
 
   if (trace(1))
     printf("msg=%s\n", g->Message);
@@ -237,7 +238,7 @@ bool CntOpenTable(PGLOBAL g, PTDB tdbp, MODE mode, char *c1, char *c2,
                                         bool del, PHC)
   {
   char   *p;
-  int     n;
+  int     i, n;
   bool    rcop= true;
   PCOL    colp;
 //PCOLUMN cp;
@@ -268,7 +269,7 @@ bool CntOpenTable(PGLOBAL g, PTDB tdbp, MODE mode, char *c1, char *c2,
 
 			if (!colp && !(mode == MODE_INSERT && tdbp->IsSpecial(p))) {
 				if (g->Message[0] == 0)
-					snprintf(g->Message, sizeof(g->Message), MSG(COL_ISNOT_TABLE), p, tdbp->GetName());
+					sprintf(g->Message, MSG(COL_ISNOT_TABLE), p, tdbp->GetName());
 
 				throw 1;
 			} // endif colp
@@ -276,7 +277,7 @@ bool CntOpenTable(PGLOBAL g, PTDB tdbp, MODE mode, char *c1, char *c2,
 			n = strlen(p) + 1;
 		} // endfor p
 
-		for (colp = tdbp->GetColumns(); colp; colp = colp->GetNext()) {
+		for (i = 0, colp = tdbp->GetColumns(); colp; i++, colp = colp->GetNext()) {
 			if (colp->InitValue(g))
 				throw 2;
 
@@ -297,7 +298,7 @@ bool CntOpenTable(PGLOBAL g, PTDB tdbp, MODE mode, char *c1, char *c2,
 			PTDBASE utp;
 
 			if (!(utp = (PTDBASE)tdbp->Duplicate(g))) {
-				snprintf(g->Message, sizeof(g->Message), MSG(INV_UPDT_TABLE), tdbp->GetName());
+				sprintf(g->Message, MSG(INV_UPDT_TABLE), tdbp->GetName());
 				throw 4;
 			} // endif tp
 
@@ -310,7 +311,7 @@ bool CntOpenTable(PGLOBAL g, PTDB tdbp, MODE mode, char *c1, char *c2,
 				n = strlen(p) + 1;
 			} // endfor p
 
-			for (colp = utp->GetColumns(); colp; colp = colp->GetNext()) {
+			for (i = 0, colp = utp->GetColumns(); colp; i++, colp = colp->GetNext()) {
 				if (colp->InitValue(g))
 					throw 5;
 
@@ -354,6 +355,7 @@ bool CntOpenTable(PGLOBAL g, PTDB tdbp, MODE mode, char *c1, char *c2,
 		} // endif mode
 
 		rcop = false;
+
 	} catch (int n) {
 		if (trace(1))
 			htrc("Exception %d: %s\n", n, g->Message);
@@ -546,8 +548,7 @@ int CntCloseTable(PGLOBAL g, PTDB tdbp, bool nox, bool abort)
 	if (!tdbp)
 		return rc;                           // Nothing to do
 	else if (tdbp->GetUse() != USE_OPEN) {
-		if (tdbp->GetAmType() == TYPE_AM_XML ||
-			tdbp->GetAmType() == TYPE_AM_JSN)
+		if (tdbp->GetAmType() == TYPE_AM_XML)
 			tdbp->CloseDB(g);                  // Opened by GetMaxSize
 
 		return rc;
@@ -627,7 +628,7 @@ int CntIndexInit(PGLOBAL g, PTDB ptdb, int id, bool sorted)
   if (!ptdb)
     return -1;
   else if (!ptdb->GetDef()->Indexable()) {
-    snprintf(g->Message, sizeof(g->Message), MSG(TABLE_NO_INDEX), ptdb->GetName());
+    sprintf(g->Message, MSG(TABLE_NO_INDEX), ptdb->GetName());
     return 0;
   } else if (ptdb->GetDef()->Indexable() == 3) {
     return 1;
@@ -645,6 +646,7 @@ int CntIndexInit(PGLOBAL g, PTDB ptdb, int id, bool sorted)
 //    return 0;
 
   if (tdbp->GetKindex())
+  {
     if (((XXBASE*)tdbp->GetKindex())->GetID() == id) {
       tdbp->GetKindex()->Reset();                // Same index
       return (tdbp->GetKindex()->IsMul()) ? 2 : 1;
@@ -652,13 +654,14 @@ int CntIndexInit(PGLOBAL g, PTDB ptdb, int id, bool sorted)
       tdbp->GetKindex()->Close();
       tdbp->SetKindex(NULL);
     } // endif colp
+  }
 
   for (xdp= dfp->GetIndx(); xdp; xdp= xdp->GetNext())
     if (xdp->GetID() == id)
       break;
 
   if (!xdp) {
-    snprintf(g->Message, sizeof(g->Message), "Wrong index ID %d", id);
+    sprintf(g->Message, "Wrong index ID %d", id);
     return 0;
     } // endif xdp
 
@@ -717,7 +720,7 @@ RCODE CntIndexRead(PGLOBAL g, PTDB ptdb, OPVAL op,
     x= ptdb->GetDef()->Indexable();
 
   if (!x) {
-    snprintf(g->Message, sizeof(g->Message), MSG(TABLE_NO_INDEX), ptdb->GetName());
+    sprintf(g->Message, MSG(TABLE_NO_INDEX), ptdb->GetName());
     return RC_FX;
   } else if (x == 2) {
     // Remote index. Only used in read mode
@@ -740,13 +743,13 @@ RCODE CntIndexRead(PGLOBAL g, PTDB ptdb, OPVAL op,
   // Set reference values and index operator
   if (!tdbp->GetLink() || !tdbp->GetKindex()) {
 //  if (!tdbp->To_Xdp) {
-      snprintf(g->Message, sizeof(g->Message), "Index not initialized for table %s", tdbp->GetName());
+      sprintf(g->Message, "Index not initialized for table %s", tdbp->GetName());
       return RC_FX;
 #if 0
       } // endif !To_Xdp
     // Now it's time to make the dynamic index
     if (tdbp->InitialyzeIndex(g, NULL, false)) {
-      snprintf(g->Message, sizeof(g->Message), "Fail to make dynamic index %s", 
+      sprintf(g->Message, "Fail to make dynamic index %s", 
                           tdbp->To_Xdp->GetName());
       return RC_FX;
       } // endif MakeDynamicIndex
@@ -786,10 +789,10 @@ RCODE CntIndexRead(PGLOBAL g, PTDB ptdb, OPVAL op,
 
         if (rcb) {
           if (tdbp->RowNumber(g))
-            snprintf(g->Message, sizeof(g->Message), "Out of range value for column %s at row %d",
+            sprintf(g->Message, "Out of range value for column %s at row %d",
                     colp->GetName(), tdbp->RowNumber(g));
           else
-            snprintf(g->Message, sizeof(g->Message), "Out of range value for column %s",
+            sprintf(g->Message, "Out of range value for column %s",
                     colp->GetName());
 
           PushWarning(g, tdbp);
@@ -848,7 +851,7 @@ int CntIndexRange(PGLOBAL g, PTDB ptdb, const uchar* *key, uint *len,
   x= ptdb->GetDef()->Indexable();
 
   if (!x) {
-    snprintf(g->Message, sizeof(g->Message), MSG(TABLE_NO_INDEX), ptdb->GetName());
+    sprintf(g->Message, MSG(TABLE_NO_INDEX), ptdb->GetName());
     DBUG_PRINT("Range", ("%s", g->Message));
     return -1;
   } else if (x == 2) {
@@ -868,7 +871,7 @@ int CntIndexRange(PGLOBAL g, PTDB ptdb, const uchar* *key, uint *len,
 
   if (!tdbp->GetKindex() || !tdbp->GetLink()) {
     if (!tdbp->GetXdp()) {
-      snprintf(g->Message, sizeof(g->Message), "Index not initialized for table %s", tdbp->GetName());
+      sprintf(g->Message, "Index not initialized for table %s", tdbp->GetName());
       DBUG_PRINT("Range", ("%s", g->Message));
       return -1;
     } else       // Dynamic index
@@ -909,11 +912,11 @@ int CntIndexRange(PGLOBAL g, PTDB ptdb, const uchar* *key, uint *len,
 
           if (rcb) {
             if (tdbp->RowNumber(g))
-              snprintf(g->Message, sizeof(g->Message),
+              sprintf(g->Message,
                       "Out of range value for column %s at row %d",
                       colp->GetName(), tdbp->RowNumber(g));
             else
-              snprintf(g->Message, sizeof(g->Message), "Out of range value for column %s",
+              sprintf(g->Message, "Out of range value for column %s",
                       colp->GetName());
 
             PushWarning(g, tdbp);
