@@ -22,12 +22,12 @@
 /*  Include relevant sections of the System header files.              */
 /***********************************************************************/
 #include "my_global.h"
-#if defined(_WIN32)
+#if defined(__WIN__)
 #include <io.h>
 #include <fcntl.h>
 //#include <errno.h>
 //#include <windows.h>
-#else   // !_WIN32
+#else   // !__WIN__
 #if defined(UNIX)
 #include <errno.h>
 #include <unistd.h>
@@ -35,7 +35,7 @@
 //#include <io.h>
 #endif  // !UNIX
 //#include <fcntl.h>
-#endif  // !_WIN32
+#endif  // !__WIN__
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
@@ -49,7 +49,6 @@
 #include "global.h"
 #include "plgdbsem.h"
 #include "filamdbf.h"
-#include "filamzip.h"
 #include "tabdos.h"
 #include "valblk.h"
 #define  NO_FUNC
@@ -140,7 +139,7 @@ static int dbfhead(PGLOBAL g, FILE *file, PCSZ fn, DBFHEADER *buf)
   if (fread(buf, HEADLEN, 1, file) != 1) {
     strcpy(g->Message, MSG(NO_READ_32));
     return RC_NF;
-  } // endif fread
+    } // endif fread
 
   // Check first byte to be sure of .dbf type
   if ((buf->Version & 0x03) != DBFTYPE) {
@@ -150,30 +149,30 @@ static int dbfhead(PGLOBAL g, FILE *file, PCSZ fn, DBFHEADER *buf)
     if ((buf->Version & 0x30) == 0x30) {
       strcpy(g->Message, MSG(FOXPRO_FILE));
       dbc = 264;             // FoxPro database container
-    } // endif Version
+      } // endif Version
 
   } else
     strcpy(g->Message, MSG(DBASE_FILE));
 
   // Check last byte(s) of header
   if (fseek(file, buf->Headlen() - dbc, SEEK_SET) != 0) {
-    snprintf(g->Message, sizeof(g->Message), MSG(BAD_HEADER), fn);
+    sprintf(g->Message, MSG(BAD_HEADER), fn);
     return RC_FX;
-  } // endif fseek
+    } // endif fseek
 
   if (fread(&endmark, 2, 1, file) != 1) {
     strcpy(g->Message, MSG(BAD_HEAD_END));
     return RC_FX;
-  } // endif fread
+    } // endif fread
 
   // Some files have just 1D others have 1D00 following fields
   if (endmark[0] != EOH && endmark[1] != EOH) {
-    snprintf(g->Message, sizeof(g->Message), MSG(NO_0DH_HEAD), dbc);
+    sprintf(g->Message, MSG(NO_0DH_HEAD), dbc);
 
     if (rc == RC_OK)
       return RC_FX;
 
-  } // endif endmark
+    } // endif endmark
 
   // Calculate here the number of fields while we have the dbc info
   buf->SetFields((buf->Headlen() - dbc - 1) / 32);
@@ -181,58 +180,13 @@ static int dbfhead(PGLOBAL g, FILE *file, PCSZ fn, DBFHEADER *buf)
   return rc;
   } // end of dbfhead
 
-/****************************************************************************/
-/*  dbfields: Analyze a DBF header and set the table fields number.         */
-/*  Parameters:                                                             */
-/*      PGLOBAL g       -- pointer to the CONNECT Global structure          */
-/*      DBFHEADER *hdrp -- pointer to _dbfheader structure                  */
-/*  Returns:                                                                */
-/*      RC_OK, RC_INFO, or RC_FX if error.                                  */
-/****************************************************************************/
-static int dbfields(PGLOBAL g, DBFHEADER* hdrp)
-{
-	char* endmark;
-	int   dbc = 2, rc = RC_OK;
-
-	*g->Message = '\0';
-
-	// Check first byte to be sure of .dbf type
-	if ((hdrp->Version & 0x03) != DBFTYPE) {
-		strcpy(g->Message, MSG(NOT_A_DBF_FILE));
-		rc = RC_INFO;
-
-		if ((hdrp->Version & 0x30) == 0x30) {
-			strcpy(g->Message, MSG(FOXPRO_FILE));
-			dbc = 264;             // FoxPro database container
-		} // endif Version
-
-	} else
-		strcpy(g->Message, MSG(DBASE_FILE));
-
-	// Check last byte(s) of header
-	endmark = (char*)hdrp + hdrp->Headlen() - dbc;
-
-	// Some headers just have 1D others have 1D00 following fields
-	if (endmark[0] != EOH && endmark[1] != EOH) {
-		snprintf(g->Message, sizeof(g->Message), MSG(NO_0DH_HEAD), dbc);
-
-		if (rc == RC_OK)
-			return RC_FX;
-
-	} // endif endmark
-
-	// Calculate here the number of fields while we have the dbc info
-	hdrp->SetFields((hdrp->Headlen() - dbc - 1) / 32);
-	return rc;
-} // end of dbfields
-
 /* -------------------------- Function DBFColumns ------------------------- */
 
 /****************************************************************************/
 /*  DBFColumns: constructs the result blocks containing the description     */
 /*  of all the columns of a DBF file that will be retrieved by #GetData.    */
 /****************************************************************************/
-PQRYRES DBFColumns(PGLOBAL g, PCSZ dp, PCSZ fn, PTOS topt, bool info)
+PQRYRES DBFColumns(PGLOBAL g, PCSZ dp, PCSZ fn, bool info)
   {
   int  buftyp[] = {TYPE_STRING, TYPE_SHORT, TYPE_STRING,
                    TYPE_INT,    TYPE_INT,   TYPE_SHORT};
@@ -242,12 +196,10 @@ PQRYRES DBFColumns(PGLOBAL g, PCSZ dp, PCSZ fn, PTOS topt, bool info)
   char       buf[2], filename[_MAX_PATH];
   int        ncol = sizeof(buftyp) / sizeof(int);
   int        rc, type, len, field, fields;
-  bool       bad, mul;
-	PCSZ       target, pwd;
-  DBFHEADER  mainhead, *hp = NULL;
-	DESCRIPTOR thisfield, *tfp;
-	FILE      *infile = NULL;
-	UNZIPUTL  *zutp = NULL;
+  bool       bad;
+  DBFHEADER  mainhead;
+  DESCRIPTOR thisfield;
+  FILE      *infile = NULL;
   PQRYRES    qrp;
   PCOLRES    crp;
 
@@ -265,55 +217,21 @@ PQRYRES DBFColumns(PGLOBAL g, PCSZ dp, PCSZ fn, PTOS topt, bool info)
     /************************************************************************/
     PlugSetPath(filename, fn, dp);
 
-		if (topt->zipped) {
-			target = GetStringTableOption(g, topt, "Entry", NULL);
-			mul = (target && *target) ? strchr(target, '*') || strchr(target, '?')
-				                        : false;
-			mul = GetBooleanTableOption(g, topt, "Mulentries", mul);
+    if (!(infile= global_fopen(g, MSGID_CANNOT_OPEN, filename, "rb")))
+      return NULL;
 
-			if (mul) {
-				strcpy(g->Message, "Cannot find column definition for multiple entries");
-				return NULL;
-			} // endif Multiple
+    /************************************************************************/
+    /*  Get the first 32 bytes of the header.                               */
+    /************************************************************************/
+    if ((rc = dbfhead(g, infile, filename, &mainhead)) == RC_FX) {
+      fclose(infile);
+      return NULL;
+      } // endif dbfhead
 
-			pwd = GetStringTableOption(g, topt, "Password", NULL);
-			zutp = new(g) UNZIPUTL(target, pwd, mul);
-
-			if (!zutp->OpenTable(g, MODE_READ, filename))
-				hp = (DBFHEADER*)zutp->memory;
-			else
-				return NULL;
-
-			/**********************************************************************/
-			/*  Set the table fields number.                                      */
-			/**********************************************************************/
-			if ((rc = dbfields(g, hp)) == RC_FX) {
-				zutp->close();
-				return NULL;
-			} // endif dbfields
-
-			tfp = (DESCRIPTOR*)hp;
-		} else {
-			if (!(infile = global_fopen(g, MSGID_CANNOT_OPEN, filename, "rb")))
-				return NULL;
-		  else
-			  hp = &mainhead;
-
-			/**********************************************************************/
-			/*  Get the first 32 bytes of the header.                             */
-			/**********************************************************************/
-			if ((rc = dbfhead(g, infile, filename, hp)) == RC_FX) {
-				fclose(infile);
-				return NULL;
-			} // endif dbfhead
-
-			tfp = &thisfield;
-		} // endif zipped
-
-		/************************************************************************/
-		/*  Get the number of the table fields.                                 */
-		/************************************************************************/
-		fields = hp->Fields();
+    /************************************************************************/
+    /*  Allocate the structures used to refer to the result set.            */
+    /************************************************************************/
+    fields = mainhead.Fields();
   } else
     fields = 0;
 
@@ -323,21 +241,19 @@ PQRYRES DBFColumns(PGLOBAL g, PCSZ dp, PCSZ fn, PTOS topt, bool info)
   if (info || !qrp) {
   	if (infile)
       fclose(infile);
-		else if (zutp)
-			zutp->close();
       
     return qrp;
-  } // endif info
+    } // endif info
 
   if (trace(1)) {
     htrc("Structure of %s\n", filename);
     htrc("headlen=%hd reclen=%hd degree=%d\n",
-          hp->Headlen(), hp->Reclen(), fields);
-    htrc("flags(iem)=%d,%d,%d cp=%d\n", hp->Incompleteflag,
-          hp->Encryptflag, hp->Mdxflag, hp->Language);
+          mainhead.Headlen(), mainhead.Reclen(), fields);
+    htrc("flags(iem)=%d,%d,%d cp=%d\n", mainhead.Incompleteflag,
+          mainhead.Encryptflag, mainhead.Mdxflag, mainhead.Language);
     htrc("%hd records, last changed %02d/%02d/%d\n",
-          hp->Records(), hp->Filedate[1], hp->Filedate[2],
-          hp->Filedate[0] + (hp->Filedate[0] <= 30) ? 2000 : 1900);
+          mainhead.Records(), mainhead.Filedate[1], mainhead.Filedate[2],
+          mainhead.Filedate[0] + (mainhead.Filedate[0] <= 30) ? 2000 : 1900);
     htrc("Field    Type  Offset  Len  Dec  Set  Mdx\n");
     } // endif trace
 
@@ -349,24 +265,21 @@ PQRYRES DBFColumns(PGLOBAL g, PCSZ dp, PCSZ fn, PTOS topt, bool info)
   for (field = 0; field < fields; field++) {
     bad = FALSE;
 
-		if (topt->zipped) {
-			tfp = (DESCRIPTOR*)((char*)tfp + HEADLEN);
-		} else if (fread(tfp, HEADLEN, 1, infile) != 1) {
-      snprintf(g->Message, sizeof(g->Message), MSG(ERR_READING_REC), field+1, fn);
+    if (fread(&thisfield, HEADLEN, 1, infile) != 1) {
+      sprintf(g->Message, MSG(ERR_READING_REC), field+1, fn);
       goto err;
-    } // endif fread
-
-    len = tfp->Length;
+    } else
+      len = thisfield.Length;
 
     if (trace(1))
       htrc("%-11s %c  %6ld  %3d   %2d  %3d  %3d\n",
-           tfp->Name, tfp->Type, tfp->Offset, len,
-           tfp->Decimals, tfp->Setfield, tfp->Mdxfield);
+           thisfield.Name, thisfield.Type, thisfield.Offset, len,
+           thisfield.Decimals, thisfield.Setfield, thisfield.Mdxfield);
 
     /************************************************************************/
     /*  Now get the results into blocks.                                    */
     /************************************************************************/
-    switch (tfp->Type) {
+    switch (thisfield.Type) {
       case 'C':                      // Characters
       case 'L':                      // Logical 'T' or 'F' or space
 				type = TYPE_STRING;
@@ -381,7 +294,7 @@ PQRYRES DBFColumns(PGLOBAL g, PCSZ dp, PCSZ fn, PTOS topt, bool info)
 			//	type = TYPE_INT;
 			//	break;
       case 'N':
-        type = (tfp->Decimals) ? TYPE_DOUBLE
+        type = (thisfield.Decimals) ? TYPE_DOUBLE
              : (len > 10) ? TYPE_BIGINT : TYPE_INT;
         break;
       case 'F':											 // Float
@@ -393,8 +306,8 @@ PQRYRES DBFColumns(PGLOBAL g, PCSZ dp, PCSZ fn, PTOS topt, bool info)
         break;
       default:
         if (!info) {
-          snprintf(g->Message, sizeof(g->Message), MSG(BAD_DBF_TYPE), tfp->Type
-                                               , tfp->Name);
+          sprintf(g->Message, MSG(BAD_DBF_TYPE), thisfield.Type
+                                               , thisfield.Name);
           goto err;
           } // endif info
 
@@ -403,31 +316,27 @@ PQRYRES DBFColumns(PGLOBAL g, PCSZ dp, PCSZ fn, PTOS topt, bool info)
       } // endswitch Type
 
     crp = qrp->Colresp;                    // Column Name
-    crp->Kdata->SetValue(tfp->Name, field);
+    crp->Kdata->SetValue(thisfield.Name, field);
     crp = crp->Next;                       // Data Type
     crp->Kdata->SetValue((int)type, field);
     crp = crp->Next;                       // Type Name
 
     if (bad) {
-      buf[0] = tfp->Type;
+      buf[0] = thisfield.Type;
       crp->Kdata->SetValue(buf, field);
     } else
       crp->Kdata->SetValue(GetTypeName(type), field);
 
     crp = crp->Next;                       // Precision
-    crp->Kdata->SetValue((int)tfp->Length, field);
+    crp->Kdata->SetValue((int)thisfield.Length, field);
     crp = crp->Next;                       // Length
-    crp->Kdata->SetValue((int)tfp->Length, field);
+    crp->Kdata->SetValue((int)thisfield.Length, field);
     crp = crp->Next;                       // Scale (precision)
-    crp->Kdata->SetValue((int)tfp->Decimals, field);
+    crp->Kdata->SetValue((int)thisfield.Decimals, field);
     } // endfor field
 
   qrp->Nblin = field;
-
-	if (infile)
-		fclose(infile);
-	else if (zutp)
-		zutp->close();
+  fclose(infile);
 
 #if 0
   if (info) {
@@ -438,11 +347,11 @@ PQRYRES DBFColumns(PGLOBAL g, PCSZ dp, PCSZ fn, PTOS topt, bool info)
 
     sprintf(buf,
       "Ver=%02x ncol=%hu nlin=%u lrecl=%hu headlen=%hu date=%02d/%02d/%02d",
-      hp->Version, fields, hp->Records, hp->Reclen,
-      hp->Headlen, hp->Filedate[0], hp->Filedate[1],
-      hp->Filedate[2]);
+      mainhead.Version, fields, mainhead.Records, mainhead.Reclen,
+      mainhead.Headlen, mainhead.Filedate[0], mainhead.Filedate[1],
+      mainhead.Filedate[2]);
 
-    safe_strcat(g->Message, sizeof(g->Message), buf);
+    strcat(g->Message, buf);
     } // endif info
 #endif // 0
 
@@ -451,13 +360,9 @@ PQRYRES DBFColumns(PGLOBAL g, PCSZ dp, PCSZ fn, PTOS topt, bool info)
   /**************************************************************************/
   return qrp;
 
-err:
-	if (infile)
-		fclose(infile);
-	else if (zutp)
-		zutp->close();
-
-	return NULL;
+ err:
+  fclose(infile);
+  return NULL;
   } // end of DBFColumns
 
 /* ---------------------------- Class DBFBASE ----------------------------- */
@@ -542,7 +447,7 @@ int DBFFAM::Cardinality(PGLOBAL g)
 
 		if (rln && Lrecl != rln) {
 			// This happens always on some Linux platforms
-			snprintf(g->Message, sizeof(g->Message), MSG(BAD_LRECL), Lrecl, (ushort)rln);
+			sprintf(g->Message, MSG(BAD_LRECL), Lrecl, (ushort)rln);
 
 			if (Accept) {
 				Lrecl = rln;
@@ -609,7 +514,7 @@ bool DBFFAM::OpenTableFile(PGLOBAL g)
       strcpy(opmode, "a+");
       break;
     default:
-      snprintf(g->Message, sizeof(g->Message), MSG(BAD_OPEN_MODE), mode);
+      sprintf(g->Message, MSG(BAD_OPEN_MODE), mode);
       return true;
     } // endswitch Mode
 
@@ -649,17 +554,17 @@ bool DBFFAM::AllocateBuffer(PGLOBAL g)
   To_Buf = (char*)PlugSubAlloc(g, NULL, Buflen);
 
   if (mode == MODE_INSERT) {
-#if defined(_WIN32)
+#if defined(__WIN__)
     /************************************************************************/
     /*  Now we can revert to binary mode in particular because the eventual */
     /*  writing of a new header must be done in binary mode to avoid        */
     /*  translating 0A bytes (LF) into 0D0A (CRLF) by Windows in text mode. */
     /************************************************************************/
     if (_setmode(_fileno(Stream), _O_BINARY) == -1) {
-      snprintf(g->Message, sizeof(g->Message), MSG(BIN_MODE_FAIL), strerror(errno));
+      sprintf(g->Message, MSG(BIN_MODE_FAIL), strerror(errno));
       return true;
       } // endif setmode
-#endif   // _WIN32
+#endif   // __WIN__
 
     /************************************************************************/
     /*  If this is a new file, the header must be generated.                */
@@ -685,7 +590,7 @@ bool DBFFAM::AllocateBuffer(PGLOBAL g)
           } // endif Flags
 
       if (Lrecl != reclen) {
-        snprintf(g->Message, sizeof(g->Message), MSG(BAD_LRECL), Lrecl, reclen);
+        sprintf(g->Message, MSG(BAD_LRECL), Lrecl, reclen);
 
 				if (Accept) {
 					Lrecl = reclen;
@@ -727,7 +632,7 @@ bool DBFFAM::AllocateBuffer(PGLOBAL g)
             case 'D':           // Date
               break;
             default:            // Should never happen
-              snprintf(g->Message, sizeof(g->Message), MSG(BAD_DBF_TYPE),
+              sprintf(g->Message, MSG(BAD_DBF_TYPE),
                                   c, cdp->GetName());
               return true;
             } // endswitch c
@@ -741,7 +646,7 @@ bool DBFFAM::AllocateBuffer(PGLOBAL g)
 
       //  Now write the header
       if (fwrite(header, 1, hlen, Stream) != (unsigned)hlen) {
-        snprintf(g->Message, sizeof(g->Message), MSG(FWRITE_ERROR), strerror(errno));
+        sprintf(g->Message, MSG(FWRITE_ERROR), strerror(errno));
         return true;
         } // endif fwrite
 
@@ -769,7 +674,7 @@ bool DBFFAM::AllocateBuffer(PGLOBAL g)
 
     if ((rc = dbfhead(g, Stream, Tdbp->GetFile(g), &header)) == RC_OK) {
       if (Lrecl != (int)header.Reclen()) {
-        snprintf(g->Message, sizeof(g->Message), MSG(BAD_LRECL), Lrecl, header.Reclen());
+        sprintf(g->Message, MSG(BAD_LRECL), Lrecl, header.Reclen());
 
 				if (Accept) {
 					Lrecl = header.Reclen();
@@ -799,7 +704,7 @@ bool DBFFAM::AllocateBuffer(PGLOBAL g)
     rc = fseek(Stream, Headlen, SEEK_SET);
 
   if (rc) {
-    snprintf(g->Message, sizeof(g->Message), MSG(BAD_DBF_FILE), Tdbp->GetFile(g));
+    sprintf(g->Message, MSG(BAD_DBF_FILE), Tdbp->GetFile(g));
     return true;
     } // endif fseek
 
@@ -857,7 +762,7 @@ int DBFFAM::ReadBuffer(PGLOBAL g)
       break;
     default:
       if (++Nerr >= Maxerr && !Accept) {
-        snprintf(g->Message, sizeof(g->Message), MSG(BAD_DBF_REC), Tdbp->GetFile(g), GetRowID());
+        sprintf(g->Message, MSG(BAD_DBF_REC), Tdbp->GetFile(g), GetRowID());
         rc = RC_FX;
       } else
         rc = (Accept) ? RC_OK : RC_NF;
@@ -882,9 +787,9 @@ bool DBFFAM::CopyHeader(PGLOBAL g)
     if (fseek(Stream, 0, SEEK_SET))
       strcpy(g->Message, "Seek error in CopyHeader");
     else if ((n = fread(hdr, 1, hlen, Stream)) != hlen)
-      snprintf(g->Message, sizeof(g->Message), MSG(BAD_READ_NUMBER), (int) n, To_File);
+      sprintf(g->Message, MSG(BAD_READ_NUMBER), (int) n, To_File);
     else if ((n = fwrite(hdr, 1, hlen, T_Stream)) != hlen)
-      snprintf(g->Message, sizeof(g->Message), MSG(WRITE_STRERROR), To_Fbt->Fname
+      sprintf(g->Message, MSG(WRITE_STRERROR), To_Fbt->Fname
                                              , strerror(errno));
     else if (fseek(Stream, pos, SEEK_SET))
       strcpy(g->Message, "Seek error in CopyHeader");
@@ -910,16 +815,16 @@ int DBFFAM::InitDelete(PGLOBAL g, int fpos, int spos)
   if (Nrec != 1)
     strcpy(g->Message, "Cannot delete in block mode");
   else if (fseek(Stream, Headlen + fpos * Lrecl, SEEK_SET))
-    snprintf(g->Message, sizeof(g->Message), MSG(FSETPOS_ERROR), 0);
+    sprintf(g->Message, MSG(FSETPOS_ERROR), 0);
   else if (fread(To_Buf, 1, lrecl, Stream) != lrecl)
-    snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, strerror(errno));
+    sprintf(g->Message, MSG(READ_ERROR), To_File, strerror(errno));
   else
     *To_Buf = '*';
 
   if (fseek(Stream, Headlen + fpos * Lrecl, SEEK_SET))
-    snprintf(g->Message, sizeof(g->Message), MSG(FSETPOS_ERROR), 0);
+    sprintf(g->Message, MSG(FSETPOS_ERROR), 0);
   else if (fwrite(To_Buf, 1, lrecl, Stream) != lrecl)
-    snprintf(g->Message, sizeof(g->Message), MSG(FWRITE_ERROR), strerror(errno));
+    sprintf(g->Message, MSG(FWRITE_ERROR), strerror(errno));
   else
     rc = RC_NF;     // Ok, Nothing else to do 
 
@@ -936,6 +841,7 @@ int DBFFAM::DeleteRecords(PGLOBAL g, int irc)
   if (irc == RC_OK) {
     // T_Stream is the temporary stream or the table file stream itself
     if (!T_Stream)
+    {
       if (UseTemp) {
         if (OpenTempFile(g))
           return RC_FX;
@@ -945,7 +851,7 @@ int DBFFAM::DeleteRecords(PGLOBAL g, int irc)
 
       } else
         T_Stream = Stream;
-
+    }
     *Tdbp->GetLine() = '*';
     Modif++;                         // Modified line in Delete mode
     } // endif irc
@@ -1062,7 +968,7 @@ int DBMFAM::Cardinality(PGLOBAL g)
 
 		if (rln && Lrecl != rln) {
 			// This happens always on some Linux platforms
-			snprintf(g->Message, sizeof(g->Message), MSG(BAD_LRECL), Lrecl, (ushort)rln);
+			sprintf(g->Message, MSG(BAD_LRECL), Lrecl, (ushort)rln);
 
 			if (Accept) {
 				Lrecl = rln;
@@ -1115,7 +1021,7 @@ bool DBMFAM::AllocateBuffer(PGLOBAL g)
     DBFHEADER *hp = (DBFHEADER*)Memory;
 
     if (Lrecl != (int)hp->Reclen()) {
-      snprintf(g->Message, sizeof(g->Message), MSG(BAD_LRECL), Lrecl, hp->Reclen());
+      sprintf(g->Message, MSG(BAD_LRECL), Lrecl, hp->Reclen());
 
 			if (Accept) {
 				Lrecl = hp->Reclen();
@@ -1168,7 +1074,7 @@ int DBMFAM::ReadBuffer(PGLOBAL g)
       break;
     default:
       if (++Nerr >= Maxerr && !Accept) {
-        snprintf(g->Message, sizeof(g->Message), MSG(BAD_DBF_REC), Tdbp->GetFile(g), GetRowID());
+        sprintf(g->Message, MSG(BAD_DBF_REC), Tdbp->GetFile(g), GetRowID());
         rc = RC_FX;
       } else
         rc = (Accept) ? RC_OK : RC_NF;
