@@ -13,13 +13,13 @@
 //#include "sql_class.h"
 //#include "sql_time.h"
 
-#if defined(_WIN32)
+#if defined(__WIN__)
 //#include <windows.h>
-#else   // !_WIN32
+#else   // !__WIN__
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#endif  // !_WIN32
+#endif  // !__WIN__
 
 
 /***********************************************************************/
@@ -83,7 +83,7 @@ BYTE OpBmp(PGLOBAL g, OPVAL opc)
     case OP_LE: bt = 0x04; break;
     case OP_EXIST: bt = 0x00; break;
     default:
-      snprintf(g->Message, sizeof(g->Message), MSG(BAD_FILTER_OP), opc);
+      sprintf(g->Message, MSG(BAD_FILTER_OP), opc);
 			throw (int)TYPE_FILTER;
 	} // endswitch opc
 
@@ -1021,7 +1021,7 @@ bool FILTER::Convert(PGLOBAL g, bool having)
     // Special case of the LIKE operator.
     if (Opc == OP_LIKE) {
       if (!IsTypeChar((int)Test[i].B_T)) {
-        snprintf(g->Message, sizeof(g->Message), MSG(BAD_TYPE_LIKE), i, Test[i].B_T);
+        sprintf(g->Message, MSG(BAD_TYPE_LIKE), i, Test[i].B_T);
         return TRUE;
         } // endif
 
@@ -1129,11 +1129,11 @@ bool FILTER::Convert(PGLOBAL g, bool having)
 
   //  Last check to be sure all is correct.
   if (Test[0].B_T != Test[1].B_T) {
-    snprintf(g->Message, sizeof(g->Message), MSG(BAD_FILTER_CONV), Test[0].B_T, Test[1].B_T);
+    sprintf(g->Message, MSG(BAD_FILTER_CONV), Test[0].B_T, Test[1].B_T);
     return TRUE;
 //} else if (Test[0].B_T == TYPE_LIST &&
 //          ((LSTVAL*)Val(0))->GetN() != ((LSTVAL*)Val(1))->GetN()) {
-//  snprintf(g->Message, sizeof(g->Message), MSG(ROW_ARGNB_ERR),
+//  sprintf(g->Message, MSG(ROW_ARGNB_ERR),
 //          ((LSTVAL*)Val(0))->GetN(), ((LSTVAL*)Val(1))->GetN());
 //  return TRUE;
   } // endif's B_T
@@ -1220,15 +1220,19 @@ bool FILTER::Eval(PGLOBAL g)
   int     i; // n = 0;
 //PSUBQ   subp = NULL;
   PARRAY  ap = NULL;
-  PDBUSER dup = PlgGetUser(g);
+  PDBUSER dup __attribute__((unused)) = PlgGetUser(g);
 
   if (Opc <= OP_XX)
+  {
     for (i = 0; i < 2; i++)
+    {
       // Evaluate the object and eventually convert it.
       if (Arg(i)->Eval(g))
         return TRUE;
       else if (Test[i].Conv)
         Val(i)->SetValue_pval(Arg(i)->GetValue());
+    }
+  }
 
   if (trace(1))
     htrc(" Filter: op=%d type=%d %d B_T=%d %d val=%p %p\n",
@@ -1367,7 +1371,7 @@ bool FILTER::Eval(PGLOBAL g)
   return FALSE;
 
  FilterError:
-  snprintf(g->Message, sizeof(g->Message), MSG(BAD_FILTER),
+  sprintf(g->Message, MSG(BAD_FILTER),
           Opc, Test[0].B_T, Test[1].B_T, GetArgType(0), GetArgType(1));
   return TRUE;
   } // end of Eval
