@@ -6,7 +6,7 @@
 /*                                                                     */
 /* COPYRIGHT:                                                          */
 /* ----------                                                          */
-/*  (C) Copyright to the author Olivier BERTRAND          1993-2020    */
+/*  (C) Copyright to the author Olivier BERTRAND          1993-2019    */
 /*                                                                     */
 /* WHAT THIS PROGRAM DOES:                                             */
 /* -----------------------                                             */
@@ -44,7 +44,7 @@
 /*                                                                     */
 /***********************************************************************/
 #include "my_global.h"
-#if defined(_WIN32)
+#if defined(__WIN__)
 //#include <windows.h>
 #else
 #if defined(UNIX) || defined(UNIV_LINUX)
@@ -81,9 +81,9 @@
 #include "rcmsg.h"
 #endif   // NEWMSG
 
-#if defined(_WIN32)
+#if defined(__WIN__)
 extern HINSTANCE s_hModule;                   /* Saved module handle    */
-#endif   // _WIN32
+#endif   // __WIN__
 
 #if defined(XMSG)
 extern char *msg_path;
@@ -96,7 +96,7 @@ char *msglang(void);
 typedef struct {
   ushort Segsize;
   ushort Size;
-} AREASIZE;
+  } AREASIZE;
 
 ACTIVITY defActivity = {            /* Describes activity and language */
   NULL,                             /* Points to user work area(s)     */
@@ -142,12 +142,12 @@ void htrc(char const* fmt, ...)
 /*  Language points on initial language name and eventual path.        */
 /*  Return value is the pointer to the Global structure.               */
 /***********************************************************************/
-PGLOBAL PlugInit(LPCSTR Language, size_t worksize)
+PGLOBAL PlugInit(LPCSTR Language, uint worksize)
 {
 	PGLOBAL g;
 
 	if (trace(2))
-		htrc("PlugInit: Language='%s'\n",
+		htrc("PlugInit: Language='%-.256s'\n",
 			((!Language) ? "Null" : (char*)Language));
 
 	try {
@@ -158,14 +158,13 @@ PGLOBAL PlugInit(LPCSTR Language, size_t worksize)
 	} // end try/catch
 
 	g->Sarea = NULL;
-	g->Createas = false;
+	g->Createas = 0;
 	g->Alchecked = 0;
 	g->Mrr = 0;
 	g->Activityp = NULL;
 	g->Xchk = NULL;
 	g->N = 0;
 	g->More = 0;
-	g->Saved_Size = 0;
 	strcpy(g->Message, "");
 
 	/*******************************************************************/
@@ -184,7 +183,7 @@ PGLOBAL PlugInit(LPCSTR Language, size_t worksize)
 /***********************************************************************/
 /*  PlugExit: Terminate Plug operations.                               */
 /***********************************************************************/
-PGLOBAL PlugExit(PGLOBAL g)
+int PlugExit(PGLOBAL g)
 {
 	if (g) {
 		PDBUSER dup = PlgGetUser(g);
@@ -196,7 +195,7 @@ PGLOBAL PlugExit(PGLOBAL g)
 		delete g;
 	}	// endif g
 
-  return NULL;
+  return 0;
 } // end of PlugExit
 
 /***********************************************************************/
@@ -204,8 +203,8 @@ PGLOBAL PlugExit(PGLOBAL g)
 /*  Note: this routine is not really implemented for Unix.             */
 /***********************************************************************/
 LPSTR PlugRemoveType(LPSTR pBuff, LPCSTR FileName)
-{
-#if defined(_WIN32)
+  {
+#if defined(__WIN__)
   char drive[_MAX_DRIVE];
 #else
   char *drive = NULL;
@@ -217,22 +216,23 @@ LPSTR PlugRemoveType(LPSTR pBuff, LPCSTR FileName)
   _splitpath(FileName, drive, direc, fname, ftype);
 
   if (trace(2)) {
-    htrc("after _splitpath: FileName=%s\n", FileName);
-    htrc("drive=%s dir=%s fname=%s ext=%s\n",
+    htrc("after _splitpath: FileName=%-.256s\n", FileName);
+    htrc("drive=%-.256s dir=%-.256s fname=%-.256s ext=%-.256s\n",
           SVP(drive), direc, fname, ftype);
     } // endif trace
 
   _makepath(pBuff, drive, direc, fname, "");
 
   if (trace(2))
-    htrc("buff='%s'\n", pBuff);
+    htrc("buff='%-.256s'\n", pBuff);
 
   return pBuff;
-} // end of PlugRemoveType
+  } // end of PlugRemoveType
+
 
 BOOL PlugIsAbsolutePath(LPCSTR path)
 {
-#if defined(_WIN32)
+#if defined(__WIN__)
   return ((path[0] >= 'a' && path[0] <= 'z') ||
           (path[0] >= 'A' && path[0] <= 'Z')) && path[1] == ':';
 #else
@@ -245,25 +245,19 @@ BOOL PlugIsAbsolutePath(LPCSTR path)
 /*  Note: this routine is not really implemented for Unix.             */
 /***********************************************************************/
 LPCSTR PlugSetPath(LPSTR pBuff, LPCSTR prefix, LPCSTR FileName, LPCSTR defpath)
-{
+  {
   char newname[_MAX_PATH];
   char direc[_MAX_DIR], defdir[_MAX_DIR], tmpdir[_MAX_DIR];
   char fname[_MAX_FNAME];
   char ftype[_MAX_EXT];
-#if defined(_WIN32)
+#if defined(__WIN__)
   char drive[_MAX_DRIVE], defdrv[_MAX_DRIVE];
 #else
   char *drive = NULL, *defdrv = NULL;
 #endif
 
 	if (trace(2))
-		htrc("prefix=%s fn=%s path=%s\n", prefix, FileName, defpath);
-
-	if (strlen(FileName) >= _MAX_PATH)
-	{
-		*pBuff= 0; /* Hope this is treated as error of some kind*/
-		return FileName;
-	}
+		htrc("prefix=%-.256s fn=%-.256s path=%-.256s\n", prefix, FileName, defpath);
 
   if (!strncmp(FileName, "//", 2) || !strncmp(FileName, "\\\\", 2)) {
     strcpy(pBuff, FileName);       // Remote file
@@ -276,18 +270,18 @@ LPCSTR PlugSetPath(LPSTR pBuff, LPCSTR prefix, LPCSTR FileName, LPCSTR defpath)
     return pBuff;
   } // endif
   
-#if !defined(_WIN32)
+#if !defined(__WIN__)
   if (*FileName == '~') {
     if (_fullpath(pBuff, FileName, _MAX_PATH)) {
       if (trace(2))
-        htrc("pbuff='%s'\n", pBuff);
+        htrc("pbuff='%-.256s'\n", pBuff);
 
      return pBuff;
     } else
       return FileName;     // Error, return unchanged name
       
     } // endif FileName  
-#endif   // !_WIN32
+#endif   // !__WIN__
   
   if (prefix && strcmp(prefix, ".") && !PlugIsAbsolutePath(defpath))
   {
@@ -315,12 +309,12 @@ LPCSTR PlugSetPath(LPSTR pBuff, LPCSTR prefix, LPCSTR FileName, LPCSTR defpath)
   _splitpath(tmpdir, defdrv, defdir, NULL, NULL);
 
   if (trace(2)) {
-    htrc("after _splitpath: FileName=%s\n", FileName);
-#if defined(_WIN32)
-    htrc("drive=%s dir=%s fname=%s ext=%s\n", drive, direc, fname, ftype);
-    htrc("defdrv=%s defdir=%s\n", defdrv, defdir);
+    htrc("after _splitpath: FileName=%-.256s\n", FileName);
+#if defined(__WIN__)
+    htrc("drive=%-.256s dir=%-.256s fname=%-.256s ext=%-.256s\n", drive, direc, fname, ftype);
+    htrc("defdrv=%-.256s defdir=%-.256s\n", defdrv, defdir);
 #else
-    htrc("dir=%s fname=%s ext=%s\n", direc, fname, ftype);
+    htrc("dir=%-.256s fname=%-.256s ext=%-.256s\n", direc, fname, ftype);
 #endif
     } // endif trace
 
@@ -342,24 +336,24 @@ LPCSTR PlugSetPath(LPSTR pBuff, LPCSTR prefix, LPCSTR FileName, LPCSTR defpath)
   _makepath(newname, drive, direc, fname, ftype);
 
   if (trace(2))
-    htrc("newname='%s'\n", newname);
+    htrc("newname='%-.256s'\n", newname);
 
   if (_fullpath(pBuff, newname, _MAX_PATH)) {
     if (trace(2))
-      htrc("pbuff='%s'\n", pBuff);
+      htrc("pbuff='%-.256s'\n", pBuff);
 
     return pBuff;
   } else
     return FileName;     // Error, return unchanged name
 
-} // end of PlugSetPath
+  } // end of PlugSetPath
 
 #if defined(XMSG)
 /***********************************************************************/
 /*  PlugGetMessage: get a message from the message file.               */
 /***********************************************************************/
 char *PlugReadMessage(PGLOBAL g, int mid, char *m)
-{
+  {
   char  msgfile[_MAX_PATH], msgid[32], buff[256];
   char *msg;
   FILE *mfile = NULL;
@@ -371,22 +365,22 @@ char *PlugReadMessage(PGLOBAL g, int mid, char *m)
   PlugSetPath(msgfile, NULL, buff, msg_path);
 
   if (!(mfile = fopen(msgfile, "rt"))) {
-    snprintf(stmsg, sizeof(stmsg), "Fail to open message file %s", msgfile);
+    sprintf(stmsg, "Fail to open message file %-.256s", msgfile);
     goto err;
     } // endif mfile
 
   for (;;)
     if (!fgets(buff, 256, mfile)) {
-      snprintf(stmsg, sizeof(stmsg), "Cannot get message %d %s", mid, SVP(m));
+      sprintf(stmsg, "Cannot get message %d %-.256s", mid, SVP(m));
       goto fin;
     } else
       if (atoi(buff) == mid)
         break;
 
-  if (sscanf(buff, " %*d %.31s \"%.255[^\"]", msgid, stmsg) < 2) {
+  if (sscanf(buff, " %*d %-.256s \"%[^\"]", msgid, stmsg) < 2) {
     // Old message file
-    if (!sscanf(buff, " %*d \"%.255[^\"]", stmsg)) {
-      snprintf(stmsg, sizeof(stmsg), "Bad message file for %d %s", mid, SVP(m));
+    if (!sscanf(buff, " %*d \"%[^\"]", stmsg)) {
+      sprintf(stmsg, "Bad message file for %d %-.256s", mid, SVP(m));
       goto fin;
     } else
       m = NULL;
@@ -410,14 +404,14 @@ char *PlugReadMessage(PGLOBAL g, int mid, char *m)
     msg =  stmsg;
 
   return msg;
-} // end of PlugReadMessage
+  } // end of PlugReadMessage
 
 #elif defined(NEWMSG)
 /***********************************************************************/
 /*  PlugGetMessage: get a message from the resource string table.      */
 /***********************************************************************/
 char *PlugGetMessage(PGLOBAL g, int mid)
-{
+  {
   char *msg;
 
 #if 0 // was !defined(UNIX) && !defined(UNIV_LINUX)
@@ -425,18 +419,17 @@ char *PlugGetMessage(PGLOBAL g, int mid)
 
   if (n == 0) {
     DWORD rc = GetLastError();
-    const int BUF_SIZE= 512;
-    msg = (char*)PlugSubAlloc(g, NULL, BUF_SIZE);   // Extend buf allocation
-    n = snprintf(msg, BUF_SIZE, "Message %d, rc=%d: ", mid, rc);
+    msg = (char*)PlugSubAlloc(g, NULL, 512);   // Extend buf allocation
+    n = sprintf(msg, "Message %d, rc=%d: ", mid, rc);
     FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM |
                   FORMAT_MESSAGE_IGNORE_INSERTS, NULL, rc, 0,
-                  (LPTSTR)(msg + n), BUF_SIZE - n, NULL);
+                  (LPTSTR)(msg + n), 512 - n, NULL);
     return msg;
     } // endif n
 
 #else  // ALL
   if (!GetRcString(mid, stmsg, 200))
-    snprintf(stmsg, sizeof(stmsg) "Message %d not found", mid);
+    sprintf(stmsg, "Message %d not found", mid);
 #endif // ALL
 
   if (g) {
@@ -446,32 +439,32 @@ char *PlugGetMessage(PGLOBAL g, int mid)
     msg =  stmsg;
 
   return msg;
-} // end of PlugGetMessage
+  } // end of PlugGetMessage
 #endif     // NEWMSG
 
-#if defined(_WIN32)
+#if defined(__WIN__)
 /***********************************************************************/
 /*  Return the line length of the console screen buffer.               */
 /***********************************************************************/
 short GetLineLength(PGLOBAL g)
-{
+  {
   CONSOLE_SCREEN_BUFFER_INFO coninfo;
   HANDLE  hcons = GetStdHandle(STD_OUTPUT_HANDLE);
   BOOL    b = GetConsoleScreenBufferInfo(hcons, &coninfo);
 
   return (b) ? coninfo.dwSize.X : 0;
-} // end of GetLineLength
-#endif   // _WIN32
+  } // end of GetLineLength
+#endif   // __WIN__
 
 /***********************************************************************/
 /*  Program for memory allocation of work and language areas.          */
 /***********************************************************************/
-bool AllocSarea(PGLOBAL g, size_t size)
+bool AllocSarea(PGLOBAL g, uint size)
 {
   /*********************************************************************/
   /*  This is the allocation routine for the WIN32/UNIX/AIX version.   */
   /*********************************************************************/
-#if defined(_WIN32)
+#if defined(__WIN__)
 	if (size >= 1048576)			 // 1M
 		g->Sarea = VirtualAlloc(NULL, size, MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
 	else
@@ -479,22 +472,20 @@ bool AllocSarea(PGLOBAL g, size_t size)
 		g->Sarea = malloc(size);
 
 	if (!g->Sarea) {
-		snprintf(g->Message, sizeof(g->Message), MSG(MALLOC_ERROR), "malloc");
+		sprintf(g->Message, MSG(MALLOC_ERROR), "malloc");
 		g->Sarea_Size = 0;
-	}	else {
-    g->Sarea_Size = size;
-    PlugSubSet(g->Sarea, size);
-  } // endif Sarea
+	}	else
+		g->Sarea_Size = size;
 
 #if defined(DEVELOPMENT)
 	if (true) {
 #else
 	if (trace(8)) {
 #endif
-    if (g->Sarea) {
-      htrc("Work area of %zd allocated at %p\n", size, g->Sarea);
-    } else
-      htrc("SareaAlloc: %s\n", g->Message);
+    if (g->Sarea)
+      htrc("Work area of %u allocated at %p\n", size, g->Sarea);
+    else
+      htrc("SareaAlloc: %-.256s\n", g->Message);
 
   } // endif trace
 
@@ -507,7 +498,7 @@ bool AllocSarea(PGLOBAL g, size_t size)
 void FreeSarea(PGLOBAL g)
 {
 	if (g->Sarea) {
-#if defined(_WIN32)
+#if defined(__WIN__)
 		if (g->Sarea_Size >= 1048576)			 // 1M
 			VirtualFree(g->Sarea, 0, MEM_RELEASE);
 		else
@@ -519,7 +510,7 @@ void FreeSarea(PGLOBAL g)
 #else
 		if (trace(8))
 #endif
-			htrc("Freeing Sarea at %p size = %zd\n", g->Sarea, g->Sarea_Size);
+			htrc("Freeing Sarea at %p size = %d\n", g->Sarea, g->Sarea_Size);
 
 		g->Sarea = NULL;
 		g->Sarea_Size = 0;
@@ -533,14 +524,14 @@ void FreeSarea(PGLOBAL g)
 /*  Here there should be some verification done such as validity of    */
 /*  the address and size not larger than memory size.                  */
 /***********************************************************************/
-BOOL PlugSubSet(void *memp, size_t size)
-{
+BOOL PlugSubSet(void *memp, uint size)
+  {
   PPOOLHEADER pph = (PPOOLHEADER)memp;
 
-  pph->To_Free = (size_t)sizeof(POOLHEADER);
+  pph->To_Free = (OFFSET)sizeof(POOLHEADER);
   pph->FreeBlk = size - pph->To_Free;
   return FALSE;
-} /* end of PlugSubSet */
+  } /* end of PlugSubSet */
 
 /***********************************************************************/
 /*  Use it to export a function that do throwing.                      */
@@ -565,22 +556,22 @@ void *PlugSubAlloc(PGLOBAL g, void *memp, size_t size)
     /*******************************************************************/
     memp = g->Sarea;
 
-  size = ROUNDUP_TO_8(size);      /* Round up size to multiple of 8 */
+  size = ((size + 7) / 8) * 8;       /* Round up size to multiple of 8 */
   pph = (PPOOLHEADER)memp;
 
   if (trace(16))
-    htrc("SubAlloc in %p size=%zd used=%zd free=%zd\n",
+    htrc("SubAlloc in %p size=%d used=%d free=%d\n",
           memp, size, pph->To_Free, pph->FreeBlk);
 
-  if (size > pph->FreeBlk) {   /* Not enough memory left in pool */
+  if ((uint)size > pph->FreeBlk) {   /* Not enough memory left in pool */
     PCSZ pname = "Work";
 
-    snprintf(g->Message, sizeof(g->Message),
-      "Not enough memory in %s area for request of %zd (used=%zd free=%zd)",
-                          pname, size, pph->To_Free, pph->FreeBlk);
+    sprintf(g->Message,
+      "Not enough memory in %-.256s area for request of %u (used=%d free=%d)",
+                          pname, (uint)size, pph->To_Free, pph->FreeBlk);
 
     if (trace(1))
-      htrc("PlugSubAlloc: %s\n", g->Message);
+      htrc("PlugSubAlloc: %-.256s\n", g->Message);
 
     DoThrow(1234);
   } /* endif size OS32 code */
@@ -589,11 +580,11 @@ void *PlugSubAlloc(PGLOBAL g, void *memp, size_t size)
   /*  Do the suballocation the simplest way.                           */
   /*********************************************************************/
   memp = MakePtr(memp, pph->To_Free); /* Points to suballocated block  */
-  pph->To_Free += size;               /* New offset of pool free block */
-  pph->FreeBlk -= size;               /* New size   of pool free block */
+  pph->To_Free += (OFFSET)size;       /* New offset of pool free block */
+  pph->FreeBlk -= (uint)size;         /* New size   of pool free block */
 
   if (trace(16))
-    htrc("Done memp=%p used=%zd free=%zd\n",
+    htrc("Done memp=%p used=%d free=%d\n",
           memp, pph->To_Free, pph->FreeBlk);
 
   return (memp);
@@ -603,7 +594,7 @@ void *PlugSubAlloc(PGLOBAL g, void *memp, size_t size)
 /*  Program for sub-allocating and copying a string in a storage area. */
 /***********************************************************************/
 char *PlugDup(PGLOBAL g, const char *str)
-{
+  {
   if (str) {
     char *sm = (char*)PlugSubAlloc(g, NULL, strlen(str) + 1);
 
@@ -612,33 +603,42 @@ char *PlugDup(PGLOBAL g, const char *str)
   } else
     return NULL;
 
-} // end of PlugDup 
+  } // end of PlugDup 
 
-/*************************************************************************/
-/* This routine makes a pointer from an offset to a memory pointer.      */
-/*************************************************************************/
-void* MakePtr(void* memp, size_t offset)
-{
-  // return ((offset == 0) ? NULL : &((char*)memp)[offset]);
-  return (!offset) ? NULL : (char *)memp + offset;
-} /* end of MakePtr */
+#if 0
+/***********************************************************************/
+/* This routine suballocate a copy of the passed string.               */
+/***********************************************************************/
+char *PlugDup(PGLOBAL g, const char *str)
+  {
+  char  *buf;
+  size_t len;
 
-/*************************************************************************/
-/* This routine makes an offset from a pointer new format.               */
-/*************************************************************************/
-size_t MakeOff(void* memp, void* ptr)
-{
-  if (ptr) {
-#if defined(_DEBUG) || defined(DEVELOPMENT)
-    if (ptr <= memp) {
-      fprintf(stderr, "ptr %p <= memp %p", ptr, memp);
-      DoThrow(999);
-    } // endif ptr
-#endif   // _DEBUG  ||         DEVELOPMENT
-    return (size_t)(((char*)ptr) - ((char*)memp));
+  if (str && (len = strlen(str))) {
+    buf = (char*)PlugSubAlloc(g, NULL, len + 1);
+    strcpy(buf, str);
   } else
-    return 0;
+    buf = NULL;
 
-} /* end of MakeOff */
+  return(buf);
+  } /* end of PlugDup */
+#endif // 0
 
-/*---------------------- End of PLUGUTIL program ------------------------*/
+/***********************************************************************/
+/* This routine makes a pointer from an offset to a memory pointer.    */
+/***********************************************************************/
+void *MakePtr(void *memp, OFFSET offset)
+  {
+  return ((offset == 0) ? NULL : &((char *)memp)[offset]);
+  } /* end of MakePtr */
+
+/***********************************************************************/
+/* This routine makes an offset from a pointer new format.             */
+/***********************************************************************/
+#if 0
+OFFSET MakeOff(void *memp, void *ptr)
+  {
+  return ((!ptr) ? 0 : (OFFSET)((char *)ptr - (char *)memp));
+  } /* end of MakeOff */
+#endif
+/*--------------------- End of PLUGUTIL program -----------------------*/
