@@ -14,7 +14,7 @@
 #include "sql_class.h"
 #include "sql_servers.h"
 #include "sql_string.h"
-#if !defined(_WIN32)
+#if !defined(__WIN__)
 #include "osutil.h"
 #endif
 
@@ -142,14 +142,8 @@ bool EXTDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
 	Username = GetStringCatInfo(g, "User", NULL);
 	Password = GetStringCatInfo(g, "Password", NULL);
 
-	// Memory was Boolean, it is now integer
-	if (!(Memory = GetIntCatInfo("Memory", 0)))
-		Memory = GetBoolCatInfo("Memory", false) ? 1 : 0;
-
-	if ((Srcdef = GetStringCatInfo(g, "Srcdef", NULL))) {
+	if ((Srcdef = GetStringCatInfo(g, "Srcdef", NULL)))
 		Read_Only = true;
-		if (Memory == 2) Memory = 1;
-	}	// endif Srcdef
 
 	Qrystr = GetStringCatInfo(g, "Query_String", "?");
 	Sep = GetStringCatInfo(g, "Separator", NULL);
@@ -159,9 +153,6 @@ bool EXTDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
 	Maxerr = GetIntCatInfo("Maxerr", 0);
 	Maxres = GetIntCatInfo("Maxres", 0);
 	Quoted = GetIntCatInfo("Quoted", 0);
-	Qchar = GetStringCatInfo(g,"Qchar", NULL);
-  if (Qchar && !Quoted)
-    Quoted = 1;
 	Options = 0;
 	Cto = 0;
 	Qto = 0;
@@ -175,6 +166,10 @@ bool EXTDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
 	if (Catfunc == FNC_TABLE)
 		Tabtyp = GetStringCatInfo(g, "Tabtype", NULL);
 
+	// Memory was Boolean, it is now integer
+	if (!(Memory = GetIntCatInfo("Memory", 0)))
+		Memory = GetBoolCatInfo("Memory", false) ? 1 : 0;
+
 	Pseudo = 2;    // FILID is Ok but not ROWID
 	return false;
 } // end of DefineAM
@@ -201,7 +196,6 @@ TDBEXT::TDBEXT(EXTDEF *tdp) : TDB(tdp)
 		Cto = tdp->Cto;
 		Qto = tdp->Qto;
 		Quoted = MY_MAX(0, tdp->GetQuoted());
-		Quote = tdp->GetQchar();
 		Rows = tdp->GetElemt();
 		Memory = tdp->Memory;
 		Scrollable = tdp->Scrollable;
@@ -218,12 +212,12 @@ TDBEXT::TDBEXT(EXTDEF *tdp) : TDB(tdp)
 		Cto = 0;
 		Qto = 0;
 		Quoted = 0;
-		Quote = NULL;
 		Rows = 0;
 		Memory = 0;
 		Scrollable = false;
 	} // endif tdp
 
+	Quote = NULL;
 	Query = NULL;
 	Count = NULL;
 	//Where = NULL;
@@ -256,7 +250,6 @@ TDBEXT::TDBEXT(PTDBEXT tdbp) : TDB(tdbp)
 	Cto = tdbp->Cto;
 	Qto = tdbp->Qto;
 	Quoted = tdbp->Quoted;
-	Quote = tdbp->Quote;
 	Rows = tdbp->Rows;
 	Memory = tdbp->Memory;
 	Scrollable = tdbp->Scrollable;
@@ -291,37 +284,6 @@ int TDBEXT::Decode(PCSZ txt, char *buf, size_t n)
 	return 0;
 } // end of Decode
 
-/*
-  Count number of %s placeholders in string.
-  Returns -1 if other sprintf placeholders are found, .g %d
-*/
-static int count_placeholders(const char *fmt)
-{
-  int cnt= 0;
-  for (const char *p=fmt; *p; p++)
-  {
-    if (*p == '%')
-    {
-      switch (p[1])
-      {
-      case 's':
-        /* %s found */
-        cnt++;
-        p++;
-        break;
-      case '%':
-        /* masking char for % found */
-        p++;
-        break;
-      default:
-        /* some other placeholder found */
-        return -1;
-      }
-    }
-  }
-  return cnt;
-}
-
 /***********************************************************************/
 /*  MakeSrcdef: make the SQL statement from SRDEF option.              */
 /***********************************************************************/
@@ -346,29 +308,16 @@ bool TDBEXT::MakeSrcdef(PGLOBAL g)
 				? To_CondFil->Having : PlugDup(g, "1=1");
 		} // endif ph
 
-		int n_placeholders = count_placeholders(Srcdef);
-		if (n_placeholders < 0)
-		{
-			strcpy(g->Message, "MakeSQL: Wrong place holders specification");
-			return true;
-		}
-
-		if (!stricmp(ph, "W") && n_placeholders <= 1) {
+		if (!stricmp(ph, "W")) {
 			Query = new(g)STRING(g, strlen(Srcdef) + strlen(fil1));
 			Query->SetLength(sprintf(Query->GetStr(), Srcdef, fil1));
-		}
-		else if (!stricmp(ph, "WH") && n_placeholders <= 2)
-		{
+		} else if (!stricmp(ph, "WH")) {
 			Query = new(g)STRING(g, strlen(Srcdef) + strlen(fil1) + strlen(fil2));
 			Query->SetLength(sprintf(Query->GetStr(), Srcdef, fil1, fil2));
-		}
-		else if (!stricmp(ph, "H") && n_placeholders <= 1)
-		{
+		} else if (!stricmp(ph, "H")) {
 			Query = new(g)STRING(g, strlen(Srcdef) + strlen(fil2));
 			Query->SetLength(sprintf(Query->GetStr(), Srcdef, fil2));
-		}
-		else if (!stricmp(ph, "HW") && n_placeholders <= 2)
-		{
+		} else if (!stricmp(ph, "HW")) {
 			Query = new(g)STRING(g, strlen(Srcdef) + strlen(fil1) + strlen(fil2));
 			Query->SetLength(sprintf(Query->GetStr(), Srcdef, fil2, fil1));
 		} else {
@@ -393,10 +342,7 @@ bool TDBEXT::MakeSQL(PGLOBAL g, bool cnt)
 	char  *catp = NULL, buf[NAM_LEN * 3];
 	int    len;
 	bool   first = true;
-	PTABLE tablep = To_Table;
 	PCOL   colp;
-	char *res= NULL, *my_schema_table= NULL;
-	size_t my_len= 0;
 
 	if (Srcdef)
 		return MakeSrcdef(g);
@@ -466,37 +412,10 @@ bool TDBEXT::MakeSQL(PGLOBAL g, bool cnt)
 	Decode(TableName, buf, sizeof(buf));
 
 	if (Quote) {
-		// Tabname can have both database and table identifiers, we need to parse
-		if (res= strstr(buf, "."))
-		{
-			// Parse schema
-			my_len= res - buf + 1;
-			my_schema_table= (char *) malloc(my_len);
-			memcpy(my_schema_table, buf, my_len - 1);
-			my_schema_table[my_len - 1] = 0;
-			Query->Append(Quote);
-			Query->Append(my_schema_table);
-			Query->Append(Quote);
-			free(my_schema_table);
-			Query->Append(".");
-			// Parse table
-			my_len= strlen(buf) - my_len + 1;
-			my_schema_table= (char *) malloc(my_len + 1);
-			memcpy(my_schema_table, ++res, my_len);
-			my_schema_table[my_len] = 0;
-			Query->Append(Quote);
-			Query->Append(my_schema_table);
-			Query->Append(Quote);
-			free(my_schema_table);
-		}
-		else
-		{
-			// Put table name between identifier quotes in case in contains blanks
-			Query->Append(Quote);
-			Query->Append(buf);
-			Query->Append(Quote);
-		}
-
+		// Put table name between identifier quotes in case in contains blanks
+		Query->Append(Quote);
+		Query->Append(buf);
+		Query->Append(Quote);
 	} else
 		Query->Append(buf);
 
@@ -535,6 +454,7 @@ void TDBEXT::RemoveConst(PGLOBAL g, char *stmt)
 	int   n, nc;
 
 	while ((p = strstr(stmt, "NAME_CONST")))
+        {
 		if ((n = sscanf(p, "%*[^,],%1024[^)])%n", val, &nc))) {
 			if (trace(33))
 				htrc("p=%s\nn=%d val=%s nc=%d\n", p, n, val, nc);
@@ -558,8 +478,8 @@ void TDBEXT::RemoveConst(PGLOBAL g, char *stmt)
 
 		} else
 			break;
-
-		return;
+        }
+	return;
 } // end of RemoveConst
 
 /***********************************************************************/
@@ -647,7 +567,7 @@ bool TDBEXT::MakeCommand(PGLOBAL g)
 			strcat(stmt, body);
 
 	} else {
-		snprintf(g->Message, sizeof(g->Message), "Cannot use this %s command",
+		sprintf(g->Message, "Cannot use this %s command",
 			(Mode == MODE_UPDATE) ? "UPDATE" : "DELETE");
 		return true;
 	} // endif p
@@ -758,7 +678,7 @@ EXTCOL::EXTCOL(PEXTCOL col1, PTDB tdbp) : COLBLK(col1, tdbp)
 bool EXTCOL::SetBuffer(PGLOBAL g, PVAL value, bool ok, bool check)
 {
 	if (!(To_Val = value)) {
-		snprintf(g->Message, sizeof(g->Message), MSG(VALUE_ERROR), Name);
+		sprintf(g->Message, MSG(VALUE_ERROR), Name);
 		return true;
 	} else if (Buf_Type == value->GetType()) {
 		// Values are of the (good) column type
@@ -777,7 +697,7 @@ bool EXTCOL::SetBuffer(PGLOBAL g, PVAL value, bool ok, bool check)
 	} else {
 		// Values are not of the (good) column type
 		if (check) {
-			snprintf(g->Message, sizeof(g->Message), MSG(TYPE_VALUE_ERR), Name,
+			sprintf(g->Message, MSG(TYPE_VALUE_ERR), Name,
 				GetTypeName(Buf_Type), GetTypeName(value->GetType()));
 			return true;
 		} // endif check
