@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 2012, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2019, MariaDB Corporation.
+Copyright (c) 2017, 2020, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,7 @@ Created Apr 25, 2012 Vasil Dimov
 #include "srv0start.h"
 #include "fil0fil.h"
 #ifdef WITH_WSREP
+# include "trx0trx.h"
 # include "mysql/service_wsrep.h"
 # include "wsrep.h"
 # include "log.h"
@@ -147,12 +148,12 @@ dict_stats_recalc_pool_add(
 schedule new estimates for table and index statistics to be calculated.
 @param[in,out]	table	persistent or temporary table
 @param[in]	thd	current session */
-void dict_stats_update_if_needed(dict_table_t* table, THD* thd)
+void dict_stats_update_if_needed(dict_table_t *table, const trx_t &trx)
 #else
 /** Update the table modification counter and if necessary,
 schedule new estimates for table and index statistics to be calculated.
 @param[in,out]	table	persistent or temporary table */
-void dict_stats_update_if_needed_func(dict_table_t* table)
+void dict_stats_update_if_needed_func(dict_table_t *table)
 #endif
 {
 	ut_ad(table->stat_initialized);
@@ -179,10 +180,9 @@ void dict_stats_update_if_needed_func(dict_table_t* table)
 			generated row locks and allow BF thread
 			lock waits to be enqueued at head of waiting
 			queue. */
-			if (thd
-			    && !wsrep_thd_is_applying(thd)
-			    && wsrep_on(thd)
-			    && wsrep_thd_is_BF(thd, 0)) {
+			if (trx.is_wsrep()
+			    && !wsrep_thd_is_applying(trx.mysql_thd)
+			    && wsrep_thd_is_BF(trx.mysql_thd, 0)) {
 				WSREP_DEBUG("Avoiding background statistics"
 					    " calculation for table %s.",
 					table->name.m_name);
@@ -393,14 +393,14 @@ dict_stats_process_entry_from_recalc_pool()
 
 	mutex_exit(&dict_sys.mutex);
 
-	/* ut_time() could be expensive, the current function
+	/* time() could be expensive, the current function
 	is called once every time a table has been changed more than 10% and
 	on a system with lots of small tables, this could become hot. If we
 	find out that this is a problem, then the check below could eventually
 	be replaced with something else, though a time interval is the natural
 	approach. */
 
-	if (ut_difftime(ut_time(), table->stats_last_recalc)
+	if (difftime(time(NULL), table->stats_last_recalc)
 	    < MIN_RECALC_INTERVAL) {
 
 		/* Stats were (re)calculated not long ago. To avoid
