@@ -412,7 +412,7 @@ row_build_low(
 	ulint			type,
 	const dict_index_t*	index,
 	const rec_t*		rec,
-	const offset_t*		offsets,
+	const rec_offs*		offsets,
 	const dict_table_t*	col_table,
 	const dtuple_t*		defaults,
 	const dict_add_v_col_t*	add_v,
@@ -428,7 +428,7 @@ row_build_low(
 	byte*			buf;
 	ulint			j;
 	mem_heap_t*		tmp_heap	= NULL;
-	offset_t		offsets_[REC_OFFS_NORMAL_SIZE];
+	rec_offs		offsets_[REC_OFFS_NORMAL_SIZE];
 	rec_offs_init(offsets_);
 
 	ut_ad(index != NULL);
@@ -476,7 +476,7 @@ row_build_low(
 	}
 
 	/* Avoid a debug assertion in rec_offs_validate(). */
-	rec_offs_make_valid(copy, index, true, const_cast<offset_t*>(offsets));
+	rec_offs_make_valid(copy, index, true, const_cast<rec_offs*>(offsets));
 
 	if (!col_table) {
 		ut_ad(!col_map);
@@ -584,7 +584,7 @@ row_build_low(
 		}
 	}
 
-	rec_offs_make_valid(rec, index, true, const_cast<offset_t*>(offsets));
+	rec_offs_make_valid(rec, index, true, const_cast<rec_offs*>(offsets));
 
 	ut_ad(dtuple_check_typed(row));
 
@@ -637,7 +637,7 @@ row_build(
 					this record must be at least
 					s-latched and the latch held
 					as long as the row dtuple is used! */
-	const offset_t*		offsets,/*!< in: rec_get_offsets(rec,index)
+	const rec_offs*		offsets,/*!< in: rec_get_offsets(rec,index)
 					or NULL, in which case this function
 					will invoke rec_get_offsets() */
 	const dict_table_t*	col_table,
@@ -690,7 +690,7 @@ row_build_w_add_vcol(
 	ulint			type,
 	const dict_index_t*	index,
 	const rec_t*		rec,
-	const offset_t*		offsets,
+	const rec_offs*		offsets,
 	const dict_table_t*	col_table,
 	const dtuple_t*		defaults,
 	const dict_add_v_col_t*	add_v,
@@ -721,7 +721,7 @@ dtuple_t*
 row_rec_to_index_entry_impl(
 	const rec_t*		rec,
 	const dict_index_t*	index,
-	const offset_t*		offsets,
+	const rec_offs*		offsets,
 	mem_heap_t*		heap,
 	ulint			info_bits = 0,
 	bool			pad = false)
@@ -869,7 +869,7 @@ dtuple_t*
 row_rec_to_index_entry_low(
 	const rec_t*		rec,
 	const dict_index_t*	index,
-	const offset_t*		offsets,
+	const rec_offs*		offsets,
 	mem_heap_t*		heap)
 {
 	return row_rec_to_index_entry_impl<false>(rec, index, offsets, heap);
@@ -884,7 +884,7 @@ row_rec_to_index_entry(
 /*===================*/
 	const rec_t*		rec,	/*!< in: record in the index */
 	const dict_index_t*	index,	/*!< in: index */
-	const offset_t*		offsets,/*!< in: rec_get_offsets(rec) */
+	const rec_offs*		offsets,/*!< in: rec_get_offsets(rec) */
 	mem_heap_t*		heap)	/*!< in: memory heap from which
 					the memory needed is allocated */
 {
@@ -900,7 +900,7 @@ row_rec_to_index_entry(
 		rec, offsets);
 
 	rec_offs_make_valid(copy_rec, index, true,
-			    const_cast<offset_t*>(offsets));
+			    const_cast<rec_offs*>(offsets));
 
 	dtuple_t* entry = rec_is_alter_metadata(copy_rec, *index)
 		? row_rec_to_index_entry_impl<true,1>(
@@ -909,7 +909,7 @@ row_rec_to_index_entry(
 			copy_rec, index, offsets, heap);
 
 	rec_offs_make_valid(rec, index, true,
-			    const_cast<offset_t*>(offsets));
+			    const_cast<rec_offs*>(offsets));
 
 	dtuple_set_info_bits(entry,
 			     rec_get_info_bits(rec, rec_offs_comp(offsets)));
@@ -928,7 +928,7 @@ dtuple_t*
 row_metadata_to_tuple(
 	const rec_t*		rec,
 	const dict_index_t*	index,
-	const offset_t*		offsets,
+	const rec_offs*		offsets,
 	mem_heap_t*		heap,
 	ulint			info_bits,
 	bool			pad)
@@ -944,7 +944,7 @@ row_metadata_to_tuple(
 		rec, offsets);
 
 	rec_offs_make_valid(copy_rec, index, true,
-			    const_cast<offset_t*>(offsets));
+			    const_cast<rec_offs*>(offsets));
 
 	dtuple_t* entry = info_bits == REC_INFO_METADATA_ALTER
 		|| rec_is_alter_metadata(copy_rec, *index)
@@ -954,7 +954,7 @@ row_metadata_to_tuple(
 			copy_rec, index, offsets, heap);
 
 	rec_offs_make_valid(rec, index, true,
-			    const_cast<offset_t*>(offsets));
+			    const_cast<rec_offs*>(offsets));
 
 	dtuple_set_info_bits(entry, info_bits);
 	return entry;
@@ -994,8 +994,8 @@ row_build_row_ref(
 	ulint		clust_col_prefix_len;
 	ulint		i;
 	mem_heap_t*	tmp_heap	= NULL;
-	offset_t	offsets_[REC_OFFS_NORMAL_SIZE];
-	offset_t*	offsets		= offsets_;
+	rec_offs	offsets_[REC_OFFS_NORMAL_SIZE];
+	rec_offs*	offsets		= offsets_;
 	rec_offs_init(offsets_);
 
 	ut_ad(index != NULL);
@@ -1090,7 +1090,7 @@ row_build_row_ref_in_tuple(
 					held as long as the row
 					reference is used! */
 	const dict_index_t*	index,	/*!< in: secondary index */
-	offset_t*		offsets)/*!< in: rec_get_offsets(rec, index)
+	rec_offs*		offsets)/*!< in: rec_get_offsets(rec, index)
 					or NULL */
 {
 	const dict_index_t*	clust_index;
@@ -1102,7 +1102,7 @@ row_build_row_ref_in_tuple(
 	ulint			clust_col_prefix_len;
 	ulint			i;
 	mem_heap_t*		heap		= NULL;
-	offset_t		offsets_[REC_OFFS_NORMAL_SIZE];
+	rec_offs		offsets_[REC_OFFS_NORMAL_SIZE];
 	rec_offs_init(offsets_);
 
 	ut_ad(!dict_index_is_clust(index));
