@@ -75,7 +75,7 @@ int maria_create(const char *name, enum data_file_type datafile_type,
   uint max_field_lengths, extra_header_size, column_nr;
   uint internal_table= flags & HA_CREATE_INTERNAL_TABLE;
   ulong reclength, real_reclength,min_pack_length;
-  char kfilename[FN_REFLEN], klinkname[FN_REFLEN], *klinkname_ptr= NullS;
+  char kfilename[FN_REFLEN], klinkname[FN_REFLEN], *klinkname_ptr;
   char dfilename[FN_REFLEN], dlinkname[FN_REFLEN], *dlinkname_ptr= 0;
   ulong pack_reclength;
   ulonglong tot_length,max_rows, tmp;
@@ -318,7 +318,7 @@ int maria_create(const char *name, enum data_file_type datafile_type,
   {
     options|= HA_OPTION_TMP_TABLE;
     tmp_table= TRUE;
-    create_mode|= O_NOFOLLOW | (internal_table ? 0 : O_EXCL);
+    create_mode|= O_NOFOLLOW;
     /* "CREATE TEMPORARY" tables are not crash-safe (dropped at restart) */
     ci->transactional= FALSE;
     flags&= ~HA_CREATE_PAGE_CHECKSUM;
@@ -716,10 +716,9 @@ int maria_create(const char *name, enum data_file_type datafile_type,
     share.base.extra_options|= MA_EXTRA_OPTIONS_INSERT_ORDER;
   }
 
-  share.state.state.key_file_length= MY_ALIGN(info_length, maria_block_size);
   DBUG_PRINT("info", ("info_length: %u", info_length));
   /* There are only 16 bits for the total header length. */
-  if (share.state.state.key_file_length > 65535)
+  if (info_length > 65535)
   {
     my_printf_error(HA_WRONG_CREATE_OPTION,
                     "Aria table '%s' has too many columns and/or "
@@ -776,7 +775,8 @@ int maria_create(const char *name, enum data_file_type datafile_type,
 
   maria_set_all_keys_active(share.state.key_map, keys);
 
-  share.base.keystart = share.state.state.key_file_length;
+  share.base.keystart = share.state.state.key_file_length=
+    MY_ALIGN(info_length, maria_block_size);
   share.base.max_key_block_length= maria_block_size;
   share.base.max_key_length=ALIGN_SIZE(max_key_length+4);
   share.base.records=ci->max_rows;
@@ -827,6 +827,11 @@ int maria_create(const char *name, enum data_file_type datafile_type,
     */
     share.state.skip_redo_lsn= share.state.is_of_horizon=
       share.state.create_rename_lsn= LSN_MAX;
+    /*
+      We have to mark the table as not movable as the table will contain the
+      maria_uuid and create_rename_lsn
+    */
+    share.state.changed|= STATE_NOT_MOVABLE;
   }
 
   if (datafile_type == DYNAMIC_RECORD)
@@ -886,9 +891,10 @@ int maria_create(const char *name, enum data_file_type datafile_type,
   {
     char *iext= strrchr(name, '.');
     int have_iext= iext && !strcmp(iext, MARIA_NAME_IEXT);
-    fn_format(kfilename, name, "", MARIA_NAME_IEXT, MY_UNPACK_FILENAME |
-              (internal_table ? 0 : MY_RETURN_REAL_PATH) |
+    fn_format(kfilename, name, "", MARIA_NAME_IEXT,
+              MY_UNPACK_FILENAME | MY_RETURN_REAL_PATH |
               (have_iext ? MY_REPLACE_EXT : MY_APPEND_EXT));
+    klinkname_ptr= NullS;
     /*
       Replace the current file.
       Don't sync dir now if the data file has the same path.
@@ -1163,7 +1169,6 @@ int maria_create(const char *name, enum data_file_type datafile_type,
                                   FALSE, TRUE))
       goto err;
     my_free(log_data);
-    log_data= 0;
   }
 
   if (!(flags & HA_DONT_TOUCH_DATA))
@@ -1446,6 +1451,7 @@ int _ma_update_state_lsns_sub(MARIA_SHARE *share, LSN lsn, TrID create_trid,
   uchar buf[LSN_STORE_SIZE * 3], *ptr;
   uchar trid_buff[8];
   File file= share->kfile.file;
+  DBUG_ENTER("_ma_update_state_lsns_sub");
   DBUG_ASSERT(file >= 0);
 
   if (lsn == LSN_IMPOSSIBLE)
@@ -1464,7 +1470,7 @@ int _ma_update_state_lsns_sub(MARIA_SHARE *share, LSN lsn, TrID create_trid,
                                               0].length,
                                     sizeof(log_array)/sizeof(log_array[0]),
                                     log_array, NULL, NULL)))
-      return res;
+      DBUG_RETURN(res);
   }
 
   for (ptr= buf; ptr < (buf + sizeof(buf)); ptr+= LSN_STORE_SIZE)
@@ -1497,13 +1503,13 @@ int _ma_update_state_lsns_sub(MARIA_SHARE *share, LSN lsn, TrID create_trid,
   }
   else
     lsn_store(buf, share->state.create_rename_lsn);
-  return (my_pwrite(file, buf, sizeof(buf),
-                    sizeof(share->state.header) +
-                    MARIA_FILE_CREATE_RENAME_LSN_OFFSET, MYF(MY_NABP)) ||
-          my_pwrite(file, trid_buff, sizeof(trid_buff),
-                    sizeof(share->state.header) +
-                    MARIA_FILE_CREATE_TRID_OFFSET, MYF(MY_NABP)) ||
-          (do_sync && mysql_file_sync(file, MYF(0))));
+  DBUG_RETURN(my_pwrite(file, buf, sizeof(buf),
+                        sizeof(share->state.header) +
+                        MARIA_FILE_CREATE_RENAME_LSN_OFFSET, MYF(MY_NABP)) ||
+              my_pwrite(file, trid_buff, sizeof(trid_buff),
+                        sizeof(share->state.header) +
+                        MARIA_FILE_CREATE_TRID_OFFSET, MYF(MY_NABP)) ||
+              (do_sync && mysql_file_sync(file, MYF(0))));
 }
 #if defined(_MSC_VER) && (_MSC_VER == 1310)
 #pragma optimize("",on)
