@@ -725,10 +725,11 @@ ha_myisam::ha_myisam(handlerton *hton, TABLE_SHARE *table_arg)
    can_enable_indexes(1)
 {}
 
-handler *ha_myisam::clone(const char *name, MEM_ROOT *mem_root)
+handler *ha_myisam::clone(const char *name __attribute__((unused)),
+                          MEM_ROOT *mem_root)
 {
-  ha_myisam *new_handler= static_cast <ha_myisam *>(handler::clone(name,
-                                                                   mem_root));
+  ha_myisam *new_handler=
+    static_cast <ha_myisam *>(handler::clone(file->filename, mem_root));
   if (new_handler)
     new_handler->file->state= file->state;
   return new_handler;
@@ -793,8 +794,8 @@ int ha_myisam::open(const char *name, int mode, uint test_if_locked)
     growing files. Using an open_flag instead of calling mi_extra(...
     HA_EXTRA_MMAP ...) after mi_open() has the advantage that the
     mapping is not repeated for every open, but just done on the initial
-    open, when the MyISAM share is created. Everytime the server
-    requires to open a new instance of a table it calls this method. We
+    open, when the MyISAM share is created. Every time the server
+    requires opening a new instance of a table it calls this method. We
     will always supply HA_OPEN_MMAP for a permanent table. However, the
     MyISAM storage engine will ignore this flag if this is a secondary
     open of a table that is in use by other threads already (if the
@@ -882,7 +883,7 @@ int ha_myisam::open(const char *name, int mode, uint test_if_locked)
 
   /*
     For static size rows, tell MariaDB that we will access all bytes
-    in the record when writing it.  This signals MariaDB to initalize
+    in the record when writing it.  This signals MariaDB to initialize
     the full row to ensure we don't get any errors from valgrind and
     that all bytes in the row is properly reset.
   */
@@ -1749,7 +1750,35 @@ void ha_myisam::start_bulk_insert(ha_rows rows, uint flags)
     else
     {
       my_bool all_keys= MY_TEST(flags & HA_CREATE_UNIQUE_INDEX_BY_SORT);
-      mi_disable_indexes_for_rebuild(file, rows, all_keys);
+      MYISAM_SHARE *share=file->s;
+      MI_KEYDEF    *key=share->keyinfo;
+      uint          i;
+     /*
+      Deactivate all indexes that can be recreated fast.
+      These include packed keys on which sorting will use more temporary
+      space than the max allowed file length or for which the unpacked keys
+      will take much more space than packed keys.
+      Note that 'rows' may be zero for the case when we don't know how many
+      rows we will put into the file.
+      Long Unique Index (HA_KEY_ALG_LONG_HASH) will not be disabled because
+      there unique property is enforced at the time of ha_write_row
+      (check_duplicate_long_entries). So we need active index at the time of
+      insert.
+     */
+      DBUG_ASSERT(file->state->records == 0 &&
+                  (!rows || rows >= MI_MIN_ROWS_TO_DISABLE_INDEXES));
+      for (i=0 ; i < share->base.keys ; i++,key++)
+      {
+        if (!(key->flag & (HA_SPATIAL | HA_AUTO_KEY)) &&
+            ! mi_too_big_key_for_sort(key,rows) && file->s->base.auto_key != i+1 &&
+            (all_keys || !(key->flag & HA_NOSAME)) &&
+            table->key_info[i].algorithm != HA_KEY_ALG_LONG_HASH)
+        {
+          mi_clear_key_active(share->state.key_map, i);
+          file->update|= HA_STATE_CHANGED;
+          file->create_unique_index_by_sort= all_keys;
+        }
+      }
     }
   }
   else
