@@ -38,7 +38,7 @@ extern const char *myisam_recover_names[];
 extern ulonglong myisam_recover_options;
 
 C_MODE_START
-ICP_RESULT index_cond_func_myisam(void *arg);
+check_result_t index_cond_func_myisam(void *arg);
 C_MODE_END
 
 class ha_myisam: public handler
@@ -69,7 +69,7 @@ class ha_myisam: public handler
   void change_table_ptr(TABLE *table_arg, TABLE_SHARE *share);
   int open(const char *name, int mode, uint test_if_locked);
   int close(void);
-  int write_row(uchar * buf);
+  int write_row(const uchar * buf);
   int update_row(const uchar * old_data, const uchar * new_data);
   int delete_row(const uchar * buf);
   int index_read_map(uchar *buf, const uchar *key, key_part_map keypart_map,
@@ -175,5 +175,5 @@ public:
 
 private:
   DsMrr_impl ds_mrr;
-  friend ICP_RESULT index_cond_func_myisam(void *arg);
+  friend check_result_t index_cond_func_myisam(void *arg);
 };
